﻿var regexEngList = {
    cardNumber: {
        pass: [/(?:credit-?)?card-?(?:number|num)?|cc[-_]?(?:n|num)/i],
        fail: [/buzzer|(?:access|gate)[-_ ]?code|card[-_ ]?(?:type|description|holder)|country|state|postalcode|city|address|CodeAbonne/i]
    },
    expiryMonth: {
        pass: [/month|mm/i],
        fail: [/year|summary|mm\s?\/\s?yy|C(?:C|V)(?:N(um)?|V)|CodeAbonne|loginForm|enterID|passcode|common|Username/i]
    },
    expiryYear: {
        pass: [/year|yy/i],
        fail: [/month|mm\s?\/\s?yy|CodeAbonne/i]
    },
    cscNumber: {
        pass: [/ccc|cvv|cvc|cvn|ccv|csc|verif|^(?!.*promo)(?=.*code).*/i],
        fail: [/(?:zip|prod|company|postal)[-_ ]?code|discount|passcode|CodeAbonne/i]
    },
    nameOnCard: {
        pass: [/holder|name/i],
        fail: [/placeholder|zip|first|last|company|log(?:on)|login(?:psudo|1_)username|Login(?:input|-|psudo|1_)password|username|Login(?:TextFields|_form(?:_name|_password))|account_name|contentHolderPage_(email|password)|uname|email|user-name|LogName/i]
    },
    address: {
        pass: [/street|address/i],
        fail: [/state|country/i]
    },
    suburb: {
        pass: [/city/i],
        fail: [/vatican[-_ ]?city/i]
    },
    state: {
        pass: [/state|province|region/i],
        fail: [/country|OnlineID|loginForm|user-id|password|username/i]
    },
    postcode: {
        pass: [/(?:zip|postal|postcode)(?!2|two)/i],
        fail: []
    },
    country: {
        pass: [/country/i],
        fail: []
    }
};

var regexIntList = {
    cardNumber: {
        pass: [/数/, /número[-_ ]?de[-_ ]?tarjeta/i, /numéro[-_ ]?de[-_ ]?carte/i, /kartennummer/i, /номер[-_ ]?карты/i, /卡号/, /कार्ड[-_ ]?नंबर/, /رقم[-_ ]?البطاقة/],
        fail: [/CodeAbonne/i]
    },
    expiryMonth: {
        pass: [/月/, /mes/i, /mois/i, /monat/i, /месяц/i, /अंतिम[-_ ]?माह|महीना/, /شهر/],
        fail: []
    },
    expiryYear: {
        pass: [/年/, /año/i, /année|CodeAbonne/i, /jahr/i, /год/i, /समाप्ति[-_ ]?वर्ष|साल/, /عام|سنة/],
        fail: []
    },
    cscNumber: {
        pass: [/सी[-_ ]?वी[-_ ]?वी[-_ ]?(नंबर|कोड)?/, /عدد[-_ ]?السيرة[-_ ]?الذاتية|رقم[-_ ]?الكود[-_ ]?الموجود[-_ ]?على[-_ ]?بطاقات[-_ ]?الاتمان/],
        fail: [/CodeAbonne/i]
    },
    nameOnCard: {
        pass: [/名/, /nombre/i, /prénom|nom/i, /название/i, /नाम/, /اسم/],
        fail: [/CodeAbonne/i]
    },
    address: {
        pass: [/住所|通り/, /calle/i, /rue/i, /straße/i, /улица/i, /街/, /सड़क/, /شارع/],
        fail: []
    },
    suburb: {
        pass: [/都市|シティ/, /ciudad/i, /ville/i, /stadt|großstadt/i, /город/i, /市/, /शहर/, /مدينة/],
        fail: []
    },
    state: {
        pass: [/県/, /prefectura/i, /etat/i, /staat|präfektur/i, /государство/i, /州/, /राज्य/, /دولة/],
        fail: []
    },
    postcode: {
        pass: [/郵便番号/, /código[-_ ]?postal/i, /code[-_ ]?postal/i, /postleitzahl|plz/i, /почтовый[-_ ]?индекс/i, /邮政编码/, /पिन[-_ ]?कोड/, /الرمز[-_ ]?البريدي/],
        fail: []
    },
    country: {
        pass: [/国/, /país/i, /pays/i, /land/i, /страна/i, /देश/, /بلد/],
        fail: []
    }
};

var InputTypesForMethod4 = {
    emailInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(email)$" }
            ]
        },
        type: {
            factor: 60,
            patterns: [
                { pattern: "^(email)$" }
            ]
        },
        ids: {
            factor: 30,
            patterns: [
                { pattern: "mail" }
            ]
        },
        labels: {
            factor: 40
        }
    },
    confirmEmailInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(email)$" }
            ]
        },
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(email)$" }
            ]
        },
        ids: {
            factor: 40,
            patterns: [
                { pattern: "confirm.*mail|mail.*confirm" },
                { pattern: "mail.*repeat|repeat.*mail" },
                { pattern: "mail.*again|again.*mail" },
                { pattern: "mail.*re.*enter|re.*enter.*mail" },
                { pattern: "mail.*retype|retype.*mail" },
                { pattern: "mail.*check|check.*mail" }
            ]
        },
        labels: {
            factor: 80
        }
    },
    telInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(tel)$" }
            ]
        },
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(tel)$" }
            ]
        },
        ids: {
            factor: 20,
            patterns: [
                { pattern: "tel|phone|mobile" }
            ]
        },
        labels: {
            factor: 40
        }
    },
    usernameInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(username)$" },
                { pattern: "^(nickname)$", guess: 50 }
            ]
        },
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(email)$" },
                { pattern: "^(text)$", guess: 5 }
            ]
        },
        ids: {
            factor: 20,
            patterns: [
                { pattern: "login|customer.?id|customer.?number|user|identifier" }
            ]
        },
        labels: {
            factor: 40
        }
    },
    passwordInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(current-password)$" },
                { pattern: "^(new-password)$" }
            ]
        },
        type: {
            factor: 60,
            patterns: [
                { pattern: "^(password)$" }
            ]
        },
        ids: {
            factor: 20,
            patterns: [
                { pattern: "password" }
            ]
        },
        labels: {
            factor: 40
        }
    },
    newPasswordInput: {
        autocomplete: {
            factor: 150,
            patterns: [
                { pattern: "^(new-password)$" }
            ]
        },
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(password)$" }
            ]
        },
        ids: {
            factor: 80,
            patterns: [
                { pattern: "new.?password" }
            ]
        },
        labels: {
            factor: 100
        }
    },
    confirmPasswordInput: {
        autocomplete: {
            factor: 150,
            patterns: [
                { pattern: "^(new-password)$" },
                { pattern: "^(current-password)$", guess: 50 }
            ]
        },
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(password)$" }
            ]
        },
        ids: {
            factor: 80,
            patterns: [
                { pattern: "confirm.*password|password.*confirm" },
                { pattern: "password.*repeat|repeat.*password" },
                { pattern: "password.*again|again.*password" },
                { pattern: "password.*check|check.*password", guess: 50 },
                { pattern: "password.*retype|retype.*password", guess: 50 },
                { pattern: "password.*verify|verify.*password", guess: 50 },
                { pattern: "confirm", guess: 50 }
            ]
        },
        labels: {
            factor: 100
        }
    },
    passcodeInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(one-time-code)$" }
            ]
        },
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(tel)$", guess: 50 }
            ]
        },
        ids: {
            factor: 20,
            patterns: [
                { pattern: "one.?time.*code" },
                { pattern: "totp|otp" },
                { pattern: "passcode" }
            ]
        },
        labels: {
            factor: 40
        }
    },
    nameInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(cc-name|cc-full-name)$" }
            ]
        },
        ids: {
            factor: 20,
            patterns: [
                { pattern: "cc.?name|cc.?full.?name|holder|name.?on.?card|owner|card.?name" }
            ]
        },
        labels: {
            factor: 40
        }
    },
    numberInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(cc-number)$" }
            ]
        },
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(tel)$", guess: 50 }
            ]
        },
        ids: {
            factor: 20,
            patterns: [
                { pattern: "card.?num|card.?no|cc.?num" },
                { pattern: "number", guess: 50 }
            ]
        },
        labels: {
            factor: 40
        }
    },
    cscInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(cc-csc)$" }
            ]
        },
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(password)$" },
                { pattern: "^(tel)$", guess: 50 }
            ]
        },
        ids: {
            factor: 40,
            patterns: [
                { pattern: "csc|cvc|cvn|cvv|verification|security.?code" }
            ]
        },
        labels: {
            factor: 80
        }
    },
    expMonthInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(cc-exp-month)$" },
                { pattern: "^(cc-exp)$", guess: 50 }
            ]
        },
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(tel)$", guess: 50 }
            ]
        },
        ids: {
            factor: 20,
            patterns: [
                { pattern: "month" }
            ]
        },
        labels: {
            factor: 40
        }
    },
    expYearInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(cc-exp-year)$" },
                { pattern: "^(cc-exp)$", guess: 50 }
            ]
        },
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(tel)$", guess: 50 }
            ]
        },
        ids: {
            factor: 20,
            patterns: [
                { pattern: "year" }
            ]
        },
        labels: {
            factor: 40
        }
    },
    expDateInput: {
        autocomplete: {
            factor: 100,
            patterns: [
                { pattern: "^(cc-exp)$" }
            ]
        },
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(tel)$", guess: 50 }
            ]
        },
        ids: {
            factor: 20,
            patterns: [
                { pattern: "cc.?exp|date|valid|expir" }
            ]
        },
        labels: {
            factor: 50
        }
    },
    searchInput: {
        type: {
            factor: 30,
            patterns: [
                { pattern: "^(search)$" }
            ]
        },
        ids: {
            factor: 20,
            patterns: [
                { pattern: "search|find" }
            ]
        },
        labels: {
            factor: 40
        }
    },
    subscribeInput: {
        ids: {
            factor: 40,
            patterns: [
                { pattern: "subscr" }
            ]
        },
        labels: {
            factor: 80
        }
    },
    submitButton: {
        type: {
            factor: 40,
            patterns: [
                { pattern: "^(submit)$" },
                { pattern: "^(image)$" },
                { pattern: "^(button)$", guess: 50 }
            ]
        },
        ids: {
            factor: 30,
            patterns: [
                { pattern: "submit|login|register|change" }
            ]
        },
        labels: {
            factor: 30
        },
        value: {
            factor: 40
        }
    }
};

var LabelGuessingPatternsForMethod4 = {
    emailInput: {

        EN: [
            { pattern: "e.?mail|@.*example" },
            { pattern: "@", guess: 50 }
        ],

        CS: [
            { pattern: "e.?mailem" }
        ],

        DA: [
            { pattern: "e.*mail" }
        ],

        DE: [
            { pattern: "e.*mail" }
        ],

        EL: [
            { pattern: "ΗΛΕΚΤΡΟΝΙΚΗ.*ΔΙΕΥΘΥΝΣΗ" }
        ],

        ES: [
            { pattern: "email" }
        ],

        FI: [
            { pattern: "sähköposti" }
        ],

        FR: [
            { pattern: "email" }
        ],

        HE: [
            { pattern: "אימייל" }
        ],

        HU: [
            { pattern: "email" }
        ],

        IT: [
            { pattern: "e.*mail" }
        ],

        JA: [
            { pattern: "e.*メール" }
        ],

        KO: [
            { pattern: "이메일" }
        ],

        NL: [
            { pattern: "e.*mail" }
        ],

        NO: [
            { pattern: "e.?post" }
        ],

        PL: [
            { pattern: "e.*mail" }
        ],

        PT: [
            { pattern: "email" }
        ],

        PT_BR: [
            { pattern: "o.*email" }
        ],

        RU: [
            { pattern: "эл.*почт" }
        ],

        RO: [
            { pattern: "e.*mail" }
        ],

        SV: [
            { pattern: "e.?post" }
        ],

        TR: [
            { pattern: "e.?posta" }
        ],

        UK: [
            { pattern: "електронн.*пошт" }
        ],

        ZH_CN: [
            { pattern: "电子.*邮件" }
        ],

        ZH: [
            { pattern: "電子郵件" }
        ]
    },
    confirmEmailInput: {

        EN: [
            { pattern: "confirm.*e.?mail|e.?mail.*confirm" },
            { pattern: "repeat.*e.?mail|e.?mail.*repeat" },
            { pattern: "e.?mail.*again|again.*e.?mail" },
            { pattern: "re.?enter.*e.?mail" },
            { pattern: "check.*e.?mail" },
            { pattern: "confirm", guess: 50 }
        ],

        CS: [
            { pattern: "potvrďte.*e.?mailem" },
            { pattern: "kontrola.*e.?mailem" },
            { pattern: "potvrďte", guess: 50 }
        ],

        DA: [
            { pattern: "bekræft.*e.*mail" },
            { pattern: "e.*mail.*kontrol" },
            { pattern: "bekræft", guess: 50 }
        ],

        DE: [
            { pattern: "e.*mail.*bestätigen|bestätigen.*e.*mail" },
            { pattern: "e.*mail.*wiederholen|e.*mail.*wiederholen" },
            { pattern: "e.*mail.*überprüfung" },
            { pattern: "bestätigen|wiederholen", guess: 50 }
        ],

        EL: [
            { pattern: "Επιβεβαίωση.*ΗΛΕΚΤΡΟΝΙΚΗ.*ΔΙΕΥΘΥΝΣΗ" },
            { pattern: "έλεγχος.*ΗΛΕΚΤΡΟΝΙΚΗ.*ΔΙΕΥΘΥΝΣΗ" },
            { pattern: "Επιβεβαίωση", guess: 50 }
        ],

        ES: [
            { pattern: "confirmar.*email" },
            { pattern: "confirmacion.*email" },
            { pattern: "confirmar", guess: 50 }
        ],

        FI: [
            { pattern: "vahvista.*sähköposti" },
            { pattern: "sähköposti.*tarkistus" },
            { pattern: "vahvista", guess: 50 }
        ],

        FR: [
            { pattern: "confirmez.*email" },
            { pattern: "vérification.*email" },
            { pattern: "confirmez", guess: 50 }
        ],

        HE: [
            { pattern: "אימייל*.אשר" },
            { pattern: "אימייל*.בדיקת" },
            { pattern: "סיסמאות", guess: 50 }
        ],

        HU: [
            { pattern: "email,*megerősítése" },
            { pattern: "email.*ellenőrzése" },
            { pattern: "megerősítése", guess: 50 }
        ],

        IT: [
            { pattern: "conferma.*e.*mail" },
            { pattern: "controllo.*e.*mail" },
            { pattern: "conferma", guess: 50 }
        ],

        JA: [
            { pattern: "e.*メール.*を.*認証.*する" },
            { pattern: "e.*メール.*チェック" },
            { pattern: "を.*認証.*する", guess: 50 }
        ],

        KO: [
            { pattern: "비밀번호.*이메일" },
            { pattern: "이메일.*비밀번호" },
            { pattern: "비밀번호", guess: 50 }
        ],

        NL: [
            { pattern: "bevestig.*e.*mail" },
            { pattern: "herhaal.*e.*mail" },
            { pattern: "e.*mail.*controle" },
            { pattern: "bevestig", guess: 50 }
        ],

        NO: [
            { pattern: "bekreft.*e.?post" },
            { pattern: "e.?post.*sjekk" },
            { pattern: "bekreft", guess: 50 }
        ],

        PL: [
            { pattern: "potwierdź.*e.*mail" },
            { pattern: "sprawdzanie.*e.*mail" },
            { pattern: "potwierdź", guess: 50 }
        ],

        PT: [
            { pattern: "confirme.*email" },
            { pattern: "verificação.*email" },
            { pattern: "confirme", guess: 50 }
        ],

        PT_BR: [
            { pattern: "confirme.*email" },
            { pattern: "verificação.*email" },
            { pattern: "confirmar", guess: 50 }
        ],

        RU: [
            { pattern: "подтвердите.*эл.*почт" },
            { pattern: "подтверждение.*эл.*почт" },
            { pattern: "повторите.*эл.*почт" },
            { pattern: "эл.*почт.*ещё.*раз|эл.*почт.*еще.*раз" },
            { pattern: "проверка.*эл.*почт" },
            { pattern: "подтвердит|подтверждение", guess: 50 }
        ],

        RO: [
            { pattern: "confirmă.*e.*mail" },
            { pattern: "verificare.*e.*mail" },
            { pattern: "confirmă", guess: 50 }
        ],

        SV: [
            { pattern: "bekräfta.*e.?post" },
            { pattern: "e.?post.*koll" },
            { pattern: "bekräfta", guess: 50 }
        ],

        TR: [
            { pattern: "e.?posta.*onayla" },
            { pattern: "e.?posta.*kontrolü" },
            { pattern: "onayla", guess: 50 }
        ],

        UK: [
            { pattern: "підтвердити.*електронн.*пошт|повторіть.*електронн.*пошт" },
            { pattern: "перевірка.*електронн.*пошт" },
            { pattern: "підтвердит|повторіт", guess: 50 }
        ],

        ZH_CN: [
            { pattern: "确认.*电子.*邮件" },
            { pattern: "电子.*邮件.*检查" },
            { pattern: "确认", guess: 50 }
        ],

        ZH: [
            { pattern: "確認.*電子郵件" },
            { pattern: "電子郵件.*檢查" },
            { pattern: "確認", guess: 50 }
        ]
    },
    telInput: {

        EN: [
            { pattern: "telephone|mobile" },
            { pattern: "phone.*number" },
            { pattern: "tel", guess: 50 }
        ],

        CS: [
            { pattern: "telefon|mobilní" },
            { pattern: "telefonní.*číslo" }
        ],

        DA: [
            { pattern: "telefon|mobil" },
            { pattern: "telefonnummer" }
        ],

        DE: [
            { pattern: "telefon|mobil" },
            { pattern: "telefonnummer" },
            { pattern: "vertragsnummer", guess: 50 }
        ],

        EL: [
            { pattern: "τηλέφωνο|κινητό" },
            { pattern: "τηλεφωνικό.*νούμερο" }
        ],

        ES: [
            { pattern: "teléfono|móvil" },
            { pattern: "número.*teléfono" }
        ],

        FI: [
            { pattern: "puhelin|kännykkä" },
            { pattern: "puhelinnumero" }
        ],

        FR: [
            { pattern: "téléphone|portable" },
            { pattern: "numéro.*téléphone" }
        ],

        HE: [
            { pattern: "טֵלֵפוֹן|נייד" },
            { pattern: "מספר*.טלפון" }
        ],

        HU: [
            { pattern: "telefon|mobil" },
            { pattern: "telefonszám" }
        ],

        IT: [
            { pattern: "telefono|cellulare" },
            { pattern: "numero.*telefono" }
        ],

        JA: [
            { pattern: "電話|携帯" },
            { pattern: "電話.*番号" }
        ],
        KO: [
            { pattern: "전화|휴대" },
            { pattern: "전화.*번호" }
        ],

        NL: [
            { pattern: "telefoon|mobiele" },
            { pattern: "telefoonnummer" }
        ],

        NO: [
            { pattern: "telefon|mobil" },
            { pattern: "telefonnummer" }
        ],

        PL: [
            { pattern: "telefon|komórkowy" },
            { pattern: "numer.*telefonu" }
        ],

        PT: [
            { pattern: "telefone|celular" },
            { pattern: "número.*telefone" }
        ],

        PT_BR: [
            { pattern: "telefone|celular" },
            { pattern: "número.*telefone" }
        ],

        RU: [
            { pattern: "телефон|мобильн" },
            { pattern: "номер.*телефон|телефон.*номер" }
        ],

        RO: [
            { pattern: "telefon|mobil" },
            { pattern: "numar.*telefon" }
        ],

        SV: [
            { pattern: "telefon|mobil" },
            { pattern: "telefonnummer" }
        ],

        TR: [
            { pattern: "telefon|cep" },
            { pattern: "telefon.*numarası" }
        ],

        UK: [
            { pattern: "телефон|мобільн" },
            { pattern: "номер.*телефон" }
        ],

        ZH_CN: [
            { pattern: "电话|移动" },
            { pattern: "电话.*号码" }
        ],

        ZH: [
            { pattern: "電話|移動" },
            { pattern: "電話號碼" }
        ]

    },
    usernameInput: {

        EN: [
            { pattern: "user" },
            { pattern: "login" },
            { pattern: "account" },
            { pattern: "tin" },
            { pattern: "customer", guess: 50 }
        ],

        CS: [
            { pattern: "uživatel" },
            { pattern: "přihlášení" },
            { pattern: "Účet" },
            { pattern: "zákazník", guess: 50 }
        ],

        DA: [
            { pattern: "bruger|brugernavn" },
            { pattern: "log.*på|přihlašovací" },
            { pattern: "konto" },
            { pattern: "kunde", guess: 50 }
        ],

        DE: [
            { pattern: "benutzer" },
            { pattern: "anmeldung|anmeldename" },
            { pattern: "konto" },
            { pattern: "kunde", guess: 50 },
            { pattern: "zugangsnummer" }
        ],

        EL: [
            { pattern: "χρήστης" },
            { pattern: "Σύνδεση" },
            { pattern: "λογαριασμός" },
            { pattern: "πελάτης", guess: 50 }
        ],

        ES: [
            { pattern: "usuario" },
            { pattern: "iniciar.*sesión" },
            { pattern: "cuenta" },
            { pattern: "nie" },
            { pattern: "cliente", guess: 50 }
        ],

        FI: [
            { pattern: "käyttäjä" },
            { pattern: "kirjaudu.*sisään|käyttäjätunnus" },
            { pattern: "tili" },
            { pattern: "asiakas", guess: 50 }
        ],

        FR: [
            { pattern: "utilisateur|identifiant" },
            { pattern: "s.?identifier" },
            { pattern: "compte" },
            { pattern: "client", guess: 50 }
        ],

        HE: [
            { pattern: "מִשׁתַמֵשׁ|צוֹרֵך" },
            { pattern: "התחברות" },
            { pattern: "חֶשְׁבּוֹן" },
            { pattern: "צרכן", guess: 50 }
        ],

        HU: [
            { pattern: "használó" },
            { pattern: "belépés" },
            { pattern: "számla" },
            { pattern: "vevő", guess: 50 }
        ],

        IT: [
            { pattern: "utente" },
            { pattern: "accesso" },
            { pattern: "account" },
            { pattern: "cliente", guess: 50 }
        ],

        JA: [
            { pattern: "ユーザー" },
            { pattern: "ログイン.*する" },
            { pattern: "アカウント" },
            { pattern: "お客様", guess: 50 }
        ],
        KO: [
            { pattern: "사용자" },
            { pattern: "로그인" },
            { pattern: "계정" },
            { pattern: "고객", guess: 50 }
        ],

        NL: [
            { pattern: "gebruiker" },
            { pattern: "account" },
            { pattern: "klant", guess: 50 }
        ],

        NO: [
            { pattern: "bruker" },
            { pattern: "logg.*inn" },
            { pattern: "regnskap" },
            { pattern: "kunde", guess: 50 }
        ],

        PL: [
            { pattern: "użytkownik" },
            { pattern: "zaloguj.*sie" },
            { pattern: "konto" },
            { pattern: "klient", guess: 50 }
        ],

        PT: [
            { pattern: "utilizador|usuário" },
            { pattern: "conecte.?se" },
            { pattern: "conta" },
            { pattern: "cliente", guess: 50 }
        ],

        PT_BR: [
            { pattern: "usuário" },
            { pattern: "login" },
            { pattern: "conta" },
            { pattern: "cliente", guess: 50 }
        ],

        RU: [
            { pattern: "пользовател" },
            { pattern: "логин" },
            { pattern: "учётн.*запис" },
            { pattern: "инн" },
            { pattern: "клиент", guess: 50 }
        ],

        RO: [
            { pattern: "utilizator" },
            { pattern: "autentificare" },
            { pattern: "cont" },
            { pattern: "client", guess: 50 }
        ],

        SV: [
            { pattern: "användare" },
            { pattern: "logga.*in" },
            { pattern: "konto" },
            { pattern: "kund", guess: 50 }
        ],

        TR: [
            { pattern: "kullanıcı" },
            { pattern: "oturum.*aç" },
            { pattern: "hesap" },
            { pattern: "müşteri", guess: 50 }
        ],

        UK: [
            { pattern: "користувач" },
            { pattern: "вхід" },
            { pattern: "обліков.*запис" },
            { pattern: "замовник", guess: 50 }
        ],

        ZH_CN: [
            { pattern: "邮箱" },
            { pattern: "用户" },
            { pattern: "登录" },
            { pattern: "帐户" },
            { pattern: "顾客", guess: 50 }
        ],

        ZH: [
            { pattern: "郵箱" },
            { pattern: "用戶" },
            { pattern: "登錄" },
            { pattern: "帳戶" },
            { pattern: "顧客", guess: 50 }
        ]

    },
    passwordInput: {

        EN: [
            { pattern: "password" }
        ],

        CS: [
            { pattern: "hesl" }
        ],

        DA: [
            { pattern: "kodeord|adgangskode|løsen|feltråb" }
        ],

        DE: [
            { pattern: "passwort|kennwort" }
        ],

        EL: [
            { pattern: "Κωδικός.*πρόσβασης" }
        ],

        ES: [
            { pattern: "contraseña" }
        ],

        FI: [
            { pattern: "salasana" }
        ],

        FR: [
            { pattern: "mot.*de.*passe" }
        ],

        HE: [
            { pattern: "סיסמה" }
        ],

        HU: [
            { pattern: "jelszó" }
        ],

        IT: [
            { pattern: "parola.*d.?ordine|gli.*password|password" }
        ],

        JA: [
            { pattern: "パスワード" }
        ],
        KO: [
            { pattern: "암호|비밀번호" }
        ],

        NL: [
            { pattern: "wachtwoord|paswoord|parool" }
        ],

        NO: [
            { pattern: "passord" }
        ],

        PL: [
            { pattern: "hasło" }
        ],

        PT: [
            { pattern: "senha" }
        ],

        PT_BR: [
            { pattern: "senha" }
        ],

        RU: [
            { pattern: "парол" }
        ],

        RO: [
            { pattern: "parola|parolă" }
        ],

        SV: [
            { pattern: "lösenord" }
        ],

        TR: [
            { pattern: "parola|Şifre" }
        ],

        UK: [
            { pattern: "парол" }
        ],

        ZH_CN: [
            { pattern: "密码" }
        ],

        ZH: [
            { pattern: "密碼" }
        ]

    },
    newPasswordInput: {

        EN: [
            { pattern: "new.*password" }
        ],

        CS: [
            { pattern: "nové.*heslo" }
        ],

        DA: [
            { pattern: "nyt.*kodeord" }
        ],

        DE: [
            { pattern: "neues.*kennwort|neues.*passwort" }
        ],

        EL: [
            { pattern: "Νέος.*Κωδικός" }
        ],

        ES: [
            { pattern: "nueva.*contraseña" }
        ],

        FI: [
            { pattern: "uusi.*salasana" }
        ],

        FR: [
            { pattern: "nouveau.*mot.*de.*passe" }
        ],

        HE: [
            { pattern: "חדשה\s*סיסמה" }
        ],

        HU: [
            { pattern: "új.*jelszó" }
        ],

        IT: [
            { pattern: "nuova.*password" }
        ],

        JA: [
            { pattern: "新しい.*パスワード" }
        ],

        KO: [
            { pattern: "새.*비밀번호" }
        ],

        NL: [
            { pattern: "nieuw.*wachtwoord|nieuw.*paswoord|nieuw.*parool" }
        ],

        NO: [
            { pattern: "nytt.*passord" }
        ],

        PL: [
            { pattern: "nowe.*hasło" }
        ],

        PT: [
            { pattern: "nova.*senha" }
        ],

        PT_BR: [
            { pattern: "nova.*senha" }
        ],

        RU: [
            { pattern: "новый.*парол" }
        ],

        RO: [
            { pattern: "parolă.*nouă" }
        ],

        SV: [
            { pattern: "nytt.*lösenord" }
        ],

        TR: [
            { pattern: "yeni.*Şifre" }
        ],

        UK: [
            { pattern: "новий.*парол" }
        ],

        ZH_CN: [
            { pattern: "新.*密码" }
        ],

        ZH: [
            { pattern: "新.*密碼" }
        ]

    },
    confirmPasswordInput: {

        EN: [
            { pattern: "confirm.*password|password.*confirm|repeat.*password|re.?enter.*password|again.*password|password.*again" },
            { pattern: "password.*check|check.*password", guess: 50 },
            { pattern: "confirm", guess: 50 }
        ],

        CS: [
            { pattern: "potvrďte.*hesl" },
            { pattern: "kontrola.*hesl", guess: 50 },
            { pattern: "potvrďte", guess: 50 }
        ],

        DA: [
            { pattern: "bekræft.*kodeord" },
            { pattern: "adgangskontrol", guess: 50 },
            { pattern: "bekræft", guess: 50 }
        ],

        DE: [
            { pattern: "kennwort.*bestätigen|bestätigen.*passwort" },
            { pattern: "kennwort.*wiederholen|wiederholen.*passwort" },
            { pattern: "passwortüberprüfung", guess: 50 },
            { pattern: "bestätigen|wiederholen", guess: 50 }
        ],

        EL: [
            { pattern: "Επιβεβαίωση.*Κωδικού" },
            { pattern: "έλεγχος.*κωδικού.*πρόσβασης", guess: 50 },
            { pattern: "Επιβεβαίωση", guess: 50 }
        ],

        ES: [
            { pattern: "confirmar.*contraseña" },
            { pattern: "confirmacion.*contraseña", guess: 50 },
            { pattern: "confirmar", guess: 50 }
        ],

        FI: [
            { pattern: "vahvista.*salasana" },
            { pattern: "salasanan.*tarkistus", guess: 50 },
            { pattern: "vahvista", guess: 50 }
        ],

        FR: [
            { pattern: "confirmez.*mot.*de.*passe" },
            { pattern: "vérification.*mot.*de.*passe", guess: 50 },
            { pattern: "confirmez", guess: 50 }
        ],

        HE: [
            { pattern: "סיסמה\s*אשר" },
            { pattern: "סיסמאות.*בדיקת", guess: 50 },
            { pattern: "סיסמאות", guess: 50 }
        ],

        HU: [
            { pattern: "jelszó.*megerősítése" },
            { pattern: "jelszó.*ellenőrzése", guess: 50 },
            { pattern: "megerősítése", guess: 50 }
        ],

        IT: [
            { pattern: "conferma.*password" },
            { pattern: "controllo.*password", guess: 50 },
            { pattern: "conferma", guess: 50 }
        ],

        JA: [
            { pattern: "パスワード.*を.*認証.*する" },
            { pattern: "パスワード.*チェック", guess: 50 },
            { pattern: "を.*認証.*する", guess: 50 }
        ],
        KO: [
            { pattern: "비밀번호.*확인" },
            { pattern: "비밀번호", guess: 50 }
        ],

        NL: [
            { pattern: "bevestig.*wachtwoord" },
            { pattern: "herhaal.*wachtwoord" },
            { pattern: "wachtwoordcontrole", guess: 50 },
            { pattern: "bevestig", guess: 50 }
        ],

        NO: [
            { pattern: "bekreft.*passord" },
            { pattern: "passordsjekk", guess: 50 },
            { pattern: "bekreft", guess: 50 }
        ],

        PL: [
            { pattern: "potwierdź.*hasło" },
            { pattern: "sprawdzanie.*hasła", guess: 50 },
            { pattern: "potwierdź", guess: 50 }
        ],

        PT: [
            { pattern: "confirme.*senha" },
            { pattern: "verificação.*senha", guess: 50 },
            { pattern: "confirme", guess: 50 }
        ],

        PT_BR: [
            { pattern: "confirmar.*senha" },
            { pattern: "verificação.*de.*senha", guess: 50 },
            { pattern: "confirmar", guess: 50 }
        ],

        RU: [
            { pattern: "подтвердите.*пароль|подтверждение.*пароля|повторите.*пароль" },
            { pattern: "пароль.*ещё.*раз|пароль.*еще.*раз" },
            { pattern: "проверка.*пароля", guess: 50 },
            { pattern: "подтвердит|подтверждение", guess: 50 }
        ],

        RO: [
            { pattern: "confirmă.*parola" },
            { pattern: "verificare.*parolă", guess: 50 },
            { pattern: "confirmă", guess: 50 }
        ],

        SV: [
            { pattern: "bekräfta.*lösenord" },
            { pattern: "lösenordskoll", guess: 50 },
            { pattern: "bekräfta", guess: 50 }
        ],

        TR: [
            { pattern: "Şifreyi.*onayla" },
            { pattern: "şifre.*kontrolü", guess: 50 },
            { pattern: "onayla", guess: 50 }
        ],

        UK: [
            { pattern: "підтвердити.*пароль|повторіть.*пароль" },
            { pattern: "перевірка.*пароля", guess: 50 },
            { pattern: "підтвердит|повторіт", guess: 50 }
        ],

        ZH_CN: [
            { pattern: "确认.*密码" },
            { pattern: "密码.*检查", guess: 50 },
            { pattern: "确认", guess: 50 }
        ],

        ZH: [
            { pattern: "確認.*密碼" },
            { pattern: "密碼.*檢查", guess: 50 },
            { pattern: "確認", guess: 50 }
        ]

    },
    passcodeInput: {

        EN: [
            { pattern: "one.*time.*code" },
            { pattern: "one.*time.*password" },
            { pattern: "otp" }
        ],

        CS: [
            { pattern: "jednorázový.*kód" },
            { pattern: "jednorázový.*hesl" }
        ],

        DA: [
            { pattern: "en.*gangs.*kode" },
            { pattern: "en.*gangs.*adgangskode" }
        ],

        DE: [
            { pattern: "einmal.*code" },
            { pattern: "einmal.*passwort" }
        ],

        EL: [
            { pattern: "εφάπαξ.*κωδικός" },
            { pattern: "κωδικός.*μιάς.*χρήσης" }
        ],

        ES: [
            { pattern: "código.*una.*sola.*vez" },
            { pattern: "contraseña.*un.*solo.*uso" }
        ],

        FI: [
            { pattern: "yhden.*kerran.*koodi" },
            { pattern: "yhden.*kerran.*salasana" }
        ],

        FR: [
            { pattern: "code.*unique" },
            { pattern: "code.*temporel" },
            { pattern: "mot.*de.*passe.*unique" }
        ],

        HE: [
            { pattern: "פעמי\s*חד\s*קוד" },
            { pattern: "פעמית*.חד*.סיסמה" }
        ],

        HU: [
            { pattern: "egyszeri.*kód" },
            { pattern: "alkalmi.*jelszó" }
        ],

        IT: [
            { pattern: "codice.*unico" },
            { pattern: "volta.*password" }
        ],

        JA: [
            { pattern: "ワン.*タイム.*コード" },
            { pattern: "ワン.*タイム.*パスワード" }
        ],
        KO: [
            { pattern: "일회성.*코드" },
            { pattern: "일회성.*패스워드" }
        ],

        NL: [
            { pattern: "eenmalige.*code" },
            { pattern: "eenmalig.*wachtwoord" }
        ],

        NO: [
            { pattern: "engangskode" },
            { pattern: "engangspassord" }
        ],

        PL: [
            { pattern: "kod.*jednorazowy" },
            { pattern: "jednorazowe.*hasło" }
        ],

        PT: [
            { pattern: "código.*único" },
            { pattern: "senha.*único" }
        ],

        PT_BR: [
            { pattern: "código.*único" },
            { pattern: "senha.*único" }
        ],

        RU: [
            { pattern: "одноразовый.*код" },
            { pattern: "одноразовый.*пароль" }
        ],

        RO: [
            { pattern: "cod.*unic" },
            { pattern: "parolă.*unică.*folosință" }
        ],

        SV: [
            { pattern: "engångskod" },
            { pattern: "engångslösenord" }
        ],

        TR: [
            { pattern: "bir.*defalık.*kod" },
            { pattern: "tek.*seferlik.*şifre" }
        ],

        UK: [
            { pattern: "одноразовий.*код" },
            { pattern: "одноразовий.*пароль" }
        ],

        ZH_CN: [
            { pattern: "一次.*性.*代码" },
            { pattern: "一次.*密码" }
        ],

        ZH: [
            { pattern: "一次.*性.*代碼" },
            { pattern: "一次.*密碼" }
        ]

    },
    searchInput: {

        EN: [
            { pattern: "search|find" }
        ],

        CS: [
            { pattern: "vyhledávání|nalézt" }
        ],

        DA: [
            { pattern: "søg|finde" }
        ],

        DE: [
            { pattern: "suche|finden" }
        ],

        EL: [
            { pattern: "Αναζήτηση|εύρημα" }
        ],

        ES: [
            { pattern: "buscar|encontrar" }
        ],

        FI: [
            { pattern: "hae|löytö" }
        ],

        FR: [
            { pattern: "chercher|trouver" }
        ],

        HE: [
            { pattern: "לחפש|למצוא" }
        ],

        HU: [
            { pattern: "keresés|megtalálja" }
        ],

        IT: [
            { pattern: "ricerca|trova" }
        ],

        JA: [
            { pattern: "探す|見つける" }
        ],
        KO: [
            { pattern: "검색|찾기" }
        ],

        NL: [
            { pattern: "zoeken|vind" }
        ],

        NO: [
            { pattern: "søk|finne" }
        ],

        PL: [
            { pattern: "szukaj|odnaleźć" }
        ],

        PT: [
            { pattern: "procurar|encontrar" }
        ],

        PT_BR: [
            { pattern: "procurar|encontrar" }
        ],

        RU: [
            { pattern: "поиск|найти|искать" }
        ],

        RO: [
            { pattern: "căutare|găsi" }
        ],

        SV: [
            { pattern: "sök|hitta" }
        ],

        TR: [
            { pattern: "arama|bulmak" }
        ],

        UK: [
            { pattern: "пошук|знайти" }
        ],

        ZH_CN: [
            { pattern: "搜索|找" }
        ],

        ZH: [
            { pattern: "搜索|找" }
        ]

    },
    subscribeInput: {

        EN: [
            { pattern: "subsc" }
        ],

        CS: [
            { pattern: "předplatné" }
        ],

        DA: [
            { pattern: "abonnement" }
        ],

        DE: [
            { pattern: "abonnement" }
        ],

        EL: [
            { pattern: "συνδρομή" }
        ],

        ES: [
            { pattern: "sottoscrizione" }
        ],

        FI: [
            { pattern: "tilaus" }
        ],

        FR: [
            { pattern: "abonnement" }
        ],

        HE: [
            { pattern: "בכתב*.התחיבות" }
        ],

        HU: [
            { pattern: "feliratkozás" }
        ],

        IT: [
            { pattern: "sottoscrizione" }
        ],

        JA: [
            { pattern: "サブ.*スク.*リ.*プ.*ション" }
        ],

        KO: [
            { pattern: "신청" }
        ],

        NL: [
            { pattern: "abonnement" }
        ],

        NO: [
            { pattern: "abonnement" }
        ],

        PL: [
            { pattern: "subskrypcja" }
        ],

        PT: [
            { pattern: "inscrição" }
        ],

        PT_BR: [
            { pattern: "inscrição" }
        ],

        RU: [
            { pattern: "подпис" }
        ],

        RO: [
            { pattern: "abonament" }
        ],

        SV: [
            { pattern: "prenumeration" }
        ],

        TR: [
            { pattern: "abone" }
        ],

        UK: [
            { pattern: "підпис" }
        ],

        ZH_CN: [
            { pattern: "订阅" }
        ],

        ZH: [
            { pattern: "訂閱" }
        ]

    },
    nameInput: {

        EN: [
            { pattern: "card.*holder" },
            { pattern: "name.*on.*card" },
            { pattern: "owner", guess: 50 },
            { pattern: "name", guess: 50 }
        ],

        CS: [
            { pattern: "držák.*karty" },
            { pattern: "jméno.*na.*kartě" },
            { pattern: "majitel", guess: 50 },
            { pattern: "název|jméno", guess: 50 }
        ],

        DA: [
            { pattern: "kortholder" },
            { pattern: "navn.*på.*kort" },
            { pattern: "ejer", guess: 50 },
            { pattern: "navn|ry", guess: 50 }
        ],

        DE: [
            { pattern: "kartenhalter" },
            { pattern: "name.*karte" },
            { pattern: "eigentümer", guess: 50 },
            { pattern: "name", guess: 50 }
        ],

        EL: [
            { pattern: "κάτοχος.*κάρτας" },
            { pattern: "όνομα.*στην.*κάρτα" },
            { pattern: "ιδιοκτήτης", guess: 50 },
            { pattern: "όνομα", guess: 50 }
        ],

        ES: [
            { pattern: "titular.*tarjeta" },
            { pattern: "nombre.*tarjeta" },
            { pattern: "poseedor", guess: 50 },
            { pattern: "nombre", guess: 50 }
        ],

        FI: [
            { pattern: "kortin.*haltija" },
            { pattern: "nimi.*kortilla" },
            { pattern: "omistaja", guess: 50 },
            { pattern: "nimi", guess: 50 }
        ],

        FR: [
            { pattern: "titulaire.*carte" },
            { pattern: "nom.*carte" },
            { pattern: "propriétaire", guess: 50 },
            { pattern: "nom", guess: 50 }
        ],

        HE: [
            { pattern: "כרטיס\s*מחזיק" },
            { pattern: "בכרטיס\s*שם" },
            { pattern: "בעלים", guess: 50 },
            { pattern: "שֵׁם", guess: 50 }
        ],

        HU: [
            { pattern: "kártyatartó" },
            { pattern: "kártyán.*szereplő.*név" },
            { pattern: "tulajdonos", guess: 50 },
            { pattern: "név", guess: 50 }
        ],

        IT: [
            { pattern: "titolare.*carta" },
            { pattern: "nome.*carta" },
            { pattern: "possessore", guess: 50 },
            { pattern: "nome", guess: 50 }
        ],

        JA: [
            { pattern: "カード.*ホルダー" },
            { pattern: "カード.*上の.*名前" },
            { pattern: "オーナー", guess: 50 },
            { pattern: "名前", guess: 50 }
        ],
        KO: [
            { pattern: "카드.*홀더" },
            { pattern: "카드.*이름" },
            { pattern: "소유자", guess: 50 },
            { pattern: "이름", guess: 50 }
        ],

        NL: [
            { pattern: "kaarthouder" },
            { pattern: "naam.*kaart" },
            { pattern: "bezitter", guess: 50 },
            { pattern: "naam", guess: 50 }
        ],

        NO: [
            { pattern: "kortholder" },
            { pattern: "navn.*på.*kort" },
            { pattern: "eieren", guess: 50 },
            { pattern: "navn", guess: 50 }
        ],

        PL: [
            { pattern: "posiadacz.*karty" },
            { pattern: "imię.*na.*karcie" },
            { pattern: "właściciel", guess: 50 },
            { pattern: "imię", guess: 50 }
        ],

        PT: [
            { pattern: "titular.*cartão" },
            { pattern: "nome.*cartão" },
            { pattern: "possuidor", guess: 50 },
            { pattern: "nome", guess: 50 }
        ],

        PT_BR: [
            { pattern: "porta.*cartas" },
            { pattern: "nome.*no.*cartão" },
            { pattern: "proprietário", guess: 50 },
            { pattern: "nome", guess: 50 }
        ],

        RU: [
            { pattern: "держатель.*карты" },
            { pattern: "имя.*карте" },
            { pattern: "владелец", guess: 50 },
            { pattern: "имя", guess: 50 }
        ],

        RO: [
            { pattern: "titular.*de.*card" },
            { pattern: "numele.*de.*pe.*card" },
            { pattern: "proprietar", guess: 50 },
            { pattern: "nume", guess: 50 }
        ],

        SV: [
            { pattern: "korthållare" },
            { pattern: "namn.*på.*kort" },
            { pattern: "ägare", guess: 50 },
            { pattern: "namn", guess: 50 }
        ],

        TR: [
            { pattern: "kart.*sahibi" },
            { pattern: "karttaki.*isim" },
            { pattern: "sahip", guess: 50 },
            { pattern: "isim", guess: 50 }
        ],

        UK: [
            { pattern: "власник.*картки" },
            { pattern: "Ім.*я.*на.*картці" },
            { pattern: "власник", guess: 50 },
            { pattern: "назва", guess: 50 }
        ],

        ZH_CN: [
            { pattern: "持.*卡.*人" },
            { pattern: "卡片.*上.*的.*名字" },
            { pattern: "所有者", guess: 50 },
            { pattern: "名称", guess: 50 }
        ],

        ZH: [
            { pattern: "持.*卡.*人" },
            { pattern: "卡片.*上.*的.*名字" },
            { pattern: "所有者", guess: 50 },
            { pattern: "名稱", guess: 50 }
        ]

    },
    numberInput: {

        EN: [
            { pattern: "card.*number" },
            { pattern: "card.*no", guess: 50 },
            { pattern: "card.*#", guess: 50 },
            { pattern: "number", guess: 50 }
        ],

        CS: [
            { pattern: "číslo.*karty" },
            { pattern: "číslo", guess: 50 }
        ],

        DA: [
            { pattern: "kortnummer" },
            { pattern: "nummer", guess: 50 }
        ],

        DE: [
            { pattern: "kartennummer" },
            { pattern: "nummer", guess: 50 }
        ],

        EL: [
            { pattern: "αριθμός.*κάρτας" },
            { pattern: "αριθμός", guess: 50 }
        ],

        ES: [
            { pattern: "número.*tarjeta" },
            { pattern: "número", guess: 50 }
        ],

        FI: [
            { pattern: "kortin.*numero" },
            { pattern: "määrä|numero", guess: 50 }
        ],

        FR: [
            { pattern: "numéro.*carte" },
            { pattern: "nombre", guess: 50 }
        ],

        HE: [
            { pattern: "כרטיס\s*מספר" },
            { pattern: "מספר", guess: 50 }
        ],

        HU: [
            { pattern: "kártyaszám" },
            { pattern: "szám", guess: 50 }
        ],

        IT: [
            { pattern: "numero.*carta" },
            { pattern: "numero", guess: 50 }
        ],

        JA: [
            { pattern: "カード.*番号" },
            { pattern: "数", guess: 50 }
        ],
        KO: [
            { pattern: "카드.*번호" },
            { pattern: "번호", guess: 50 }
        ],

        NL: [
            { pattern: "kaartnummer" },
            { pattern: "aantal|nummer|getal", guess: 50 }
        ],

        NO: [
            { pattern: "kortnummer" },
            { pattern: "antall", guess: 50 }
        ],

        PL: [
            { pattern: "numer.*karty" },
            { pattern: "numer", guess: 50 }
        ],

        PT: [
            { pattern: "número.*cartão" },
            { pattern: "número", guess: 50 }
        ],

        PT_BR: [
            { pattern: "número.*do.*cartão" },
            { pattern: "número", guess: 50 }
        ],

        RU: [
            { pattern: "номер.*карт" },
            { pattern: "номер", guess: 50 }
        ],

        RO: [
            { pattern: "număr.*de.*card" },
            { pattern: "număr", guess: 50 }
        ],

        SV: [
            { pattern: "kortnummer" },
            { pattern: "siffra|nummer", guess: 50 }
        ],

        TR: [
            { pattern: "kart.*numarası" },
            { pattern: "numara", guess: 50 }
        ],

        UK: [
            { pattern: "номер.*картки" },
            { pattern: "номер", guess: 50 }
        ],

        ZH_CN: [
            { pattern: "卡.*号" },
            { pattern: "数|编号|号", guess: 50 }
        ],

        ZH: [
            { pattern: "卡.*號" },
            { pattern: "數", guess: 50 }
        ]

    },
    cscInput: {

        EN: [
            { pattern: "csc|cvc|cvn|cvv" },
            { pattern: "security.*code" },
            { pattern: "verification", guess: 50 },
            { pattern: "code", guess: 50 }
        ],

        CS: [
            { pattern: "bezpečnostní.*kód" },
            { pattern: "ověření", guess: 50 },
            { pattern: "kód", guess: 50 }
        ],

        DA: [
            { pattern: "sikkerhedskode" },
            { pattern: "verifikation", guess: 50 },
            { pattern: "kode", guess: 50 }
        ],

        DE: [
            { pattern: "sicherheitscode" },
            { pattern: "nachprüfung", guess: 50 },
            { pattern: "code", guess: 50 }
        ],

        EL: [
            { pattern: "κωδικός.*ασφαλείας" },
            { pattern: "επαλήθευση", guess: 50 },
            { pattern: "κώδικας", guess: 50 }
        ],

        ES: [
            { pattern: "código.*seguridad" },
            { pattern: "verificación", guess: 50 },
            { pattern: "código", guess: 50 }
        ],

        FI: [
            { pattern: "turvakoodi" },
            { pattern: "todentaminen", guess: 50 },
            { pattern: "koodi", guess: 50 }
        ],

        FR: [
            { pattern: "code.*sécurité" },
            { pattern: "vérification", guess: 50 },
            { pattern: "code", guess: 50 }
        ],

        HE: [
            { pattern: "אבטחה\s*קוד" },
            { pattern: "אימות", guess: 50 },
            { pattern: "קוד", guess: 50 }
        ],

        HU: [
            { pattern: "biztonsági.*kód" },
            { pattern: "igazolás", guess: 50 },
            { pattern: "kód", guess: 50 }
        ],

        IT: [
            { pattern: "codice.*sicurezza" },
            { pattern: "verifica", guess: 50 },
            { pattern: "codice", guess: 50 }
        ],

        JA: [
            { pattern: "セキュリティ.*コード" },
            { pattern: "検証", guess: 50 },
            { pattern: "コード", guess: 50 }
        ],
        KO: [
            { pattern: "보안.*코드" },
            { pattern: "확인", guess: 50 },
            { pattern: "암호|코드", guess: 50 }
        ],

        NL: [
            { pattern: "beveiligingscode" },
            { pattern: "verificatie", guess: 50 },
            { pattern: "code", guess: 50 }
        ],

        NO: [
            { pattern: "sikkerhetskode" },
            { pattern: "bekreftelse", guess: 50 },
            { pattern: "kode", guess: 50 }
        ],

        PL: [
            { pattern: "kod.*bezpieczeństwa" },
            { pattern: "weryfikacja", guess: 50 },
            { pattern: "kod", guess: 50 }
        ],

        PT: [
            { pattern: "código.*segurança" },
            { pattern: "verificação", guess: 50 },
            { pattern: "código", guess: 50 }
        ],

        PT_BR: [
            { pattern: "código.*de.*segurança" },
            { pattern: "verificação", guess: 50 },
            { pattern: "código", guess: 50 }
        ],

        RU: [
            { pattern: "код.*безопасности" },
            { pattern: "проверка", guess: 50 },
            { pattern: "код", guess: 50 }
        ],

        RO: [
            { pattern: "cod.*de.*securitate" },
            { pattern: "verificare", guess: 50 },
            { pattern: "cod", guess: 50 }
        ],

        SV: [
            { pattern: "säkerhetskod" },
            { pattern: "verifiering", guess: 50 },
            { pattern: "koda", guess: 50 }
        ],

        TR: [
            { pattern: "güvenlik.*kod" },
            { pattern: "doğrulama", guess: 50 },
            { pattern: "kod", guess: 50 }
        ],

        UK: [
            { pattern: "код.*безпеки" },
            { pattern: "перевірка", guess: 50 },
            { pattern: "код", guess: 50 }
        ],

        ZH_CN: [
            { pattern: "安全.*码" },
            { pattern: "验证", guess: 50 },
            { pattern: "码", guess: 50 }
        ],

        ZH: [
            { pattern: "安全.*碼" },
            { pattern: "驗證", guess: 50 },
            { pattern: "碼", guess: 50 }
        ]

    },
    expMonthInput: {

        EN: [
            { pattern: "month" },
            { pattern: "\\bmm\\b" }
        ],

        CS: [
            { pattern: "měsíc" },
            { pattern: "\\bmm\\b" }
        ],

        DA: [
            { pattern: "måned" },
            { pattern: "\\bmm\\b" }
        ],

        DE: [
            { pattern: "monat" },
            { pattern: "\\bmm\\b" }
        ],

        EL: [
            { pattern: "μήνας" }
        ],

        ES: [
            { pattern: "mes" },
            { pattern: "\\bmm\\b" }
        ],

        FI: [
            { pattern: "kuukausi" }
        ],

        FR: [
            { pattern: "mois" },
            { pattern: "\\bmm\\b" }
        ],

        HE: [
            { pattern: "חוֹדֶשׁ|יֶרַח" }
        ],

        HU: [
            { pattern: "hónap" }
        ],

        IT: [
            { pattern: "mese" },
            { pattern: "\\bmm\\b" }
        ],

        JA: [
            { pattern: "月" }
        ],
        KO: [
            { pattern: "달" }
        ],

        NL: [
            { pattern: "maand" }
        ],

        NO: [
            { pattern: "måned" }
        ],

        PL: [
            { pattern: "miesiąc" },
            { pattern: "\\bmm\\b" }
        ],

        PT: [
            { pattern: "mês" },
            { pattern: "\\bmm\\b" }
        ],

        PT_BR: [
            { pattern: "mês" },
            { pattern: "\\bmm\\b" }
        ],

        RU: [
            { pattern: "месяц" },
            { pattern: "\\bмм\\b" }
        ],

        RO: [
            { pattern: "lună" },
            { pattern: "\\bll\\b" }
        ],

        SV: [
            { pattern: "månad" }
        ],

        TR: [
            { pattern: "ay" }
        ],

        UK: [
            { pattern: "місяць" },
            { pattern: "\\bмм\\b" }
        ],

        ZH_CN: [
            { pattern: "月" }
        ],

        ZH: [
            { pattern: "月" }
        ]

    },
    expYearInput: {

        EN: [
            { pattern: "year" },
            { pattern: "\\byy\\b" },
            { pattern: "\\byyyy\\b" }
        ],

        CS: [
            { pattern: "rok" },
            { pattern: "\\brr\\b" },
            { pattern: "\\brrrr\\b" }
        ],

        DA: [
            { pattern: "år" },
            { pattern: "\\båå\\b" },
            { pattern: "\\båååå\\b" }
        ],

        DE: [
            { pattern: "jahr" },
            { pattern: "\\bjj\\b" },
            { pattern: "\\bjjjj\\b" }
        ],

        EL: [
            { pattern: "έτος" }
        ],

        ES: [
            { pattern: "año" },
            { pattern: "\\baa\\b" },
            { pattern: "\\baaaa\\b" }
        ],

        FI: [
            { pattern: "vuosi" }
        ],

        FR: [
            { pattern: "an" },
            { pattern: "\\baa\\b" },
            { pattern: "\\baaaa\\b" }
        ],

        HE: [
            { pattern: "שָׁנָה" }
        ],

        HU: [
            { pattern: "év" }
        ],

        IT: [
            { pattern: "anno" },
            { pattern: "\\baa\\b" },
            { pattern: "\\baaaa\\b" }
        ],

        JA: [
            { pattern: "年" }
        ],
        KO: [
            { pattern: "년" }
        ],

        NL: [
            { pattern: "jaar" }
        ],

        NO: [
            { pattern: "år" }
        ],

        PL: [
            { pattern: "rok" },
            { pattern: "\\brr\\b" },
            { pattern: "\\brrrr\\b" }
        ],

        PT: [
            { pattern: "ano" },
            { pattern: "\\baa\\b" },
            { pattern: "\\baaaa\\b" }
        ],

        PT_BR: [
            { pattern: "ano" },
            { pattern: "\\baa\\b" },
            { pattern: "\\baaaa\\b" }
        ],

        RU: [
            { pattern: "год" },
            { pattern: "\\bгг\\b" },
            { pattern: "\\bгггг\\b" }
        ],

        RO: [
            { pattern: "an" },
            { pattern: "\\baa\\b" },
            { pattern: "\\baaaa\\b" }
        ],

        SV: [
            { pattern: "år" }
        ],

        TR: [
            { pattern: "yıl" }
        ],

        UK: [
            { pattern: "рік" },
            { pattern: "\\bрр\\b" },
            { pattern: "\\bррpp\\b" }
        ],

        ZH_CN: [
            { pattern: "年|年度|岁" }
        ],

        ZH: [
            { pattern: "年" }
        ]

    },
    expDateInput: {

        EN: [
            { pattern: "date" },
            { pattern: "valid" },
            { pattern: "mm.*yy", guess: 50 }
        ],

        CS: [
            { pattern: "datum" },
            { pattern: "platný" },
            { pattern: "mm.*rr", guess: 50 }
        ],

        DA: [
            { pattern: "dato|daddel" },
            { pattern: "gyldig|velbegrundet" },
            { pattern: "mm.*åå", guess: 50 }
        ],

        DE: [
            { pattern: "datum" },
            { pattern: "gültig" },
            { pattern: "mm.*jj", guess: 50 }
        ],

        EL: [
            { pattern: "ημερομηνία" },
            { pattern: "έγκυρος" }
        ],

        ES: [
            { pattern: "fecha" },
            { pattern: "válido|valedero|vigente" },
            { pattern: "mm.*aa", guess: 50 }
        ],

        FI: [
            { pattern: "päivämäärä" },
            { pattern: "pätevä" }
        ],

        FR: [
            { pattern: "date|rancard" },
            { pattern: "valide|valable" },
            { pattern: "mm.*aa", guess: 50 }
        ],

        HE: [
            { pattern: "תַאֲרִיך" },
            { pattern: "תָקֵף" }
        ],

        HU: [
            { pattern: "dátum" },
            { pattern: "érvényes" }
        ],

        IT: [
            { pattern: "data|dattero" },
            { pattern: "valido|giustificato" },
            { pattern: "mm.*aa", guess: 50 }
        ],

        JA: [
            { pattern: "日付" },
            { pattern: "有効" }
        ],
        KO: [
            { pattern: "데이트" },
            { pattern: "유효한" }
        ],

        NL: [
            { pattern: "datum|dagtekening|afspraakje" },
            { pattern: "geldig|geldend" }
        ],

        NO: [
            { pattern: "dato|daddel" },
            { pattern: "gyldig" }
        ],

        PL: [
            { pattern: "data" },
            { pattern: "ważny" },
            { pattern: "mm.*rr", guess: 50 }
        ],

        PT: [
            { pattern: "encontro|tâmara|data" },
            { pattern: "válido" },
            { pattern: "mm.*aa", guess: 50 }
        ],

        PT_BR: [
            { pattern: "data" },
            { pattern: "válido" },
            { pattern: "mm.*aa", guess: 50 }
        ],

        RU: [
            { pattern: "дата" },
            { pattern: "действительна" },
            { pattern: "мм.*гг|mm.*гг", guess: 50 }
        ],

        RO: [
            { pattern: "data" },
            { pattern: "valabil" },
            { pattern: "ll.*aa", guess: 50 }
        ],

        SV: [
            { pattern: "datum" },
            { pattern: "giltig" }
        ],

        TR: [
            { pattern: "tarih" },
            { pattern: "geçerli" }
        ],

        UK: [
            { pattern: "дата" },
            { pattern: "дійсний" }
        ],

        ZH_CN: [
            { pattern: "日期" },
            { pattern: "有效" }
        ],

        ZH: [
            { pattern: "日期" },
            { pattern: "有效" }
        ]

    },
    submitButton: {

        EN: [
            { pattern: "submit|send" },
            { pattern: "login|sign.*in|log.*in|enter|admission" },
            { pattern: "register" },
            { pattern: "change" }
        ],

        CS: [
            { pattern: "předložit|poslat" },
            { pattern: "přihlásit.*se|vstoupit|přijetí" },
            { pattern: "registrovat" },
            { pattern: "změna" }
        ],

        DA: [
            { pattern: "indsend|sende" },
            { pattern: "log.*på|log.*ind|gå.*ind|adgang" },
            { pattern: "tilmeld" },
            { pattern: "lave.*om" }
        ],

        DE: [
            { pattern: "einreichen|senden" },
            { pattern: "anmeldung|einloggen|anmeldung|eingeben|eintritt" },
            { pattern: "registrieren" },
            { pattern: "veränderung" }
        ],

        EL: [
            { pattern: "υποβάλλουν|στείλετε" },
            { pattern: "Σύνδεση|Συνδεθείτε|εισαγω|άδεια" },
            { pattern: "κανω.*ΕΓΓΡΑΦΗ" },
            { pattern: "αλλαγή" }
        ],

        ES: [
            { pattern: "enviar" },
            { pattern: "iniciar.*sesión|registrarse|entrar|admisión" },
            { pattern: "registrarse" },
            { pattern: "cambio" }
        ],

        FI: [
            { pattern: "lähetä|lähettää" },
            { pattern: "kirjaudu.*sisään|tulla.*sisään|pääsy" },
            { pattern: "rekisteröidy" },
            { pattern: "muuttaa" }
        ],

        FR: [
            { pattern: "soumettre|envoyer" },
            { pattern: "s.*identifier|se.*connecter|entrer|admission" },
            { pattern: "s.*inscrire" },
            { pattern: "changement" }
        ],

        HE: [
            { pattern: "שינוי|לִשְׁלוֹחַ" },
            { pattern: "התחברות|להתחבר|להיכנס|הוֹדָאָה" },
            { pattern: "להירשם" },
            { pattern: "שינוי" }
        ],

        HU: [
            { pattern: "beküldés|küld" },
            { pattern: "bejelentkezés|belép|belép" },
            { pattern: "regisztráció" },
            { pattern: "változás" }
        ],

        IT: [
            { pattern: "invia|spedire" },
            { pattern: "accesso|registrati|accedere|ammissione|accedi" },
            { pattern: "registrati" },
            { pattern: "modificare" }
        ],

        JA: [
            { pattern: "参加.*する|送る" },
            { pattern: "ログイン.*する|サイン.*イン|入る|入場" },
            { pattern: "登録" },
            { pattern: "変化.*する" }
        ],
        KO: [
            { pattern: "제출|보내다" },
            { pattern: "로그인|로그인|시작하다|입장" },
            { pattern: "레지스터" },
            { pattern: "변화" }
        ],

        NL: [
            { pattern: "indienen|sturen" },
            { pattern: "inloggen|log.*in|invoeren|toelating" },
            { pattern: "registreren" },
            { pattern: "verandering" }
        ],

        NO: [
            { pattern: "sende.*inn|sende" },
            { pattern: "logg.*inn|tast.*inn|adgang" },
            { pattern: "registrere" },
            { pattern: "endring" }
        ],

        PL: [
            { pattern: "zatwierdź|wysłać" },
            { pattern: "zaloguj.*sie|wchodzić|wstęp" },
            { pattern: "zarejestrować" },
            { pattern: "zmiana" }
        ],

        PT: [
            { pattern: "enviar" },
            { pattern: "сonecte.*se|assinar.*em|entrar|admissão" },
            { pattern: "registro" },
            { pattern: "mudança" }
        ],

        PT_BR: [
            { pattern: "enviar" },
            { pattern: "сonecte.*se|assinar.*em|entrar|admissão" },
            { pattern: "registro" },
            { pattern: "mudança" }
        ],

        RU: [
            { pattern: "отправить" },
            { pattern: "войти|вход|зайти" },
            { pattern: "регистр" },
            { pattern: "изменит|помен" }
        ],

        RO: [
            { pattern: "trimite" },
            { pattern: "autentificare|conectare|introduce|admitere" },
            { pattern: "inregistreaza.*te" },
            { pattern: "schimbare" }
        ],

        SV: [
            { pattern: "skicka.*in|skicka" },
            { pattern: "logga.*in|stiga.*på|tillträde" },
            { pattern: "registrera" },
            { pattern: "förändra" }
        ],

        TR: [
            { pattern: "sunmak|göndermek" },
            { pattern: "oturum.*aç|giriş|kabul" },
            { pattern: "kayıt.*ol" },
            { pattern: "değişiklik" }
        ],

        UK: [
            { pattern: "подати|надіслати" },
            { pattern: "логін|увійти|увійдіть|введіть|допуск" },
            { pattern: "реєструвати" },
            { pattern: "змінити" }
        ],

        ZH_CN: [
            { pattern: "提交|发送" },
            { pattern: "登录|登入|输入|入.*场" },
            { pattern: "寄存器" },
            { pattern: "更改" }
        ],

        ZH: [
            { pattern: "提交|發送" },
            { pattern: "登錄|登入|輸入|入場" },
            { pattern: "寄存器" },
            { pattern: "更改" }
        ]

    }
};

// ======= Detect Local Credit Card Fields ===========
function DetectLocalCreditCardFields(creditCardFields) {
    // DO NOT auto-fill on the official application site.
    var url = GetExtensionWebsiteUrl();
    var officialSite = String.ReplaceAll(url, "/{0}", "");
    if (GetWindowLocationUrl().indexOf(officialSite) > -1) {
        return creditCardFields;
    }

    var creditCardForm = {};
    $jq3_51("form").each(function () {
        if (!IsObjectNull(creditCardForm["numberInput"]) &&
            !IsObjectNull(creditCardForm["expMonthInput"]) &&
            !IsObjectNull(creditCardForm["expYearInput"])) {
            return;
        }
        var form = $jq3_51(this).get(0);
        creditCardForm = Method4_GuessInputs(form, true);
    });

    // Name on card field.
    var $nameOnCard = GetInputFieldsByAttributesOnContentPage(
        "nameOnCard", regexEngList);

    if (IsJqueryObjectNullOrBlank($nameOnCard) === true) {
        $nameOnCard = GetInputFieldsByAttributesOnContentPage(
            "nameOnCard", regexIntList);
    }

    if (IsJqueryObjectNullOrBlank($nameOnCard) === true &&
        IsObjectNull(creditCardForm["nameInput"]) === false) {
        $nameOnCard = creditCardForm["nameInput"];
    }

    if (IsObjectNull(creditCardFields.NameOnCardField) &&
        IsJqueryObjectNullOrBlank($nameOnCard) === false) {
        creditCardFields.NameOnCardField = $nameOnCard;
    }

    // Card number field.
    var $cardNumber = GetInputFieldsByAttributesOnContentPage(
        "cardNumber", regexEngList);

    if (IsJqueryObjectNullOrBlank($cardNumber) === true) {
        $cardNumber = GetInputFieldsByAttributesOnContentPage(
            "cardNumber", regexIntList);
    }

    if (IsJqueryObjectNullOrBlank($cardNumber) === true &&
        IsObjectNull(creditCardForm["numberInput"]) === false) {
        $cardNumber = creditCardForm["numberInput"];
    }

    if (IsObjectNull(creditCardFields.CardNumberField) &&
        IsJqueryObjectNullOrBlank($cardNumber) === false) {
        creditCardFields.CardNumberField = $cardNumber;
    }

    // Expiry month field.
    var $expiryMonth = GetInputFieldsByAttributesOnContentPage(
        "expiryMonth", regexEngList);

    if (IsJqueryObjectNullOrBlank($expiryMonth) === true) {
        $expiryMonth = GetInputFieldsByAttributesOnContentPage(
            "expiryMonth", regexIntList);
    }

    if (IsJqueryObjectNullOrBlank($expiryMonth) === true &&
        IsObjectNull(creditCardForm["expMonthInput"]) === false) {
        $expiryMonth = creditCardForm["expMonthInput"];
    }

    if (IsJqueryObjectNullOrBlank($expiryMonth) === true &&
        IsObjectNull(creditCardForm["expDateInput"]) === false) {
        $expiryMonth = creditCardForm["expDateInput"];
    }

    if (IsObjectNull(creditCardFields.ExpiryMonthField) &&
        IsJqueryObjectNullOrBlank($expiryMonth) === false) {
        creditCardFields.ExpiryMonthField = $expiryMonth;
    }

    // Expiry year field.
    var $expiryYear = GetInputFieldsByAttributesOnContentPage(
        "expiryYear", regexEngList);

    if (IsJqueryObjectNullOrBlank($expiryYear) === true) {
        $expiryYear = GetInputFieldsByAttributesOnContentPage(
            "expiryYear", regexIntList);
    }

    if (IsJqueryObjectNullOrBlank($expiryYear) === true &&
        IsObjectNull(creditCardForm["expYearInput"]) === false) {
        $expiryYear = creditCardForm["expYearInput"];
    }

    if (IsJqueryObjectNullOrBlank($expiryYear) === true &&
        IsObjectNull(creditCardForm["expDateInput"]) === false) {
        $expiryYear = creditCardForm["expDateInput"];
    }

    if (IsObjectNull(creditCardFields.ExpiryYearField) &&
        IsJqueryObjectNullOrBlank($expiryYear) === false) {
        creditCardFields.ExpiryYearField = $expiryYear;
    }

    // CSC number field.
    var $cscNumber = GetInputFieldsByAttributesOnContentPage(
        "cscNumber", regexEngList);

    if (IsJqueryObjectNullOrBlank($cscNumber) === true) {
        $cscNumber = GetInputFieldsByAttributesOnContentPage(
            "cscNumber", regexIntList);
    }

    if (IsJqueryObjectNullOrBlank($cscNumber) === true &&
        IsObjectNull(creditCardForm["cscInput"]) === false) {
        $cscNumber = creditCardForm["cscInput"];
    }

    if (IsObjectNull(creditCardFields.CscNumberField) &&
        IsJqueryObjectNullOrBlank($cscNumber) === false) {
        creditCardFields.CscNumberField = $cscNumber;
    }
    return creditCardFields;
};

// ========= Detect Local Address Fields =============
function DetectLocalAddressFields(addressFields) {
    // DO NOT auto-fill on the official application site.
    var url = GetExtensionWebsiteUrl();
    var officialSite = String.ReplaceAll(url, "/{0}", "");
    if (GetWindowLocationUrl().indexOf(officialSite) > -1) {
        return addressFields;
    }

    // Address field.
    var $address = GetInputFieldsByAttributesOnContentPage(
        "address", regexEngList);

    if (IsJqueryObjectNullOrBlank($address) === true) {
        $address = GetInputFieldsByAttributesOnContentPage(
            "address", regexIntList);
    }

    if (IsObjectNull(addressFields.AddressField) &&
        IsJqueryObjectNullOrBlank($address) === false) {
        addressFields.AddressField = $address;
    }

    // Suburb field.
    var $suburb = GetInputFieldsByAttributesOnContentPage(
        "suburb", regexEngList);

    if (IsJqueryObjectNullOrBlank($suburb) === true) {
        $suburb = GetInputFieldsByAttributesOnContentPage(
            "suburb", regexIntList);
    }

    if (IsObjectNull(addressFields.SuburbField) &&
        IsJqueryObjectNullOrBlank($suburb) === false) {
        addressFields.SuburbField = $suburb;
    }

    // State field.
    var $state = GetInputFieldsByAttributesOnContentPage(
        "state", regexEngList);

    if (IsJqueryObjectNullOrBlank($state) === true) {
        $state = GetInputFieldsByAttributesOnContentPage(
            "state", regexIntList);
    }

    if (IsObjectNull(addressFields.StateField) &&
        IsJqueryObjectNullOrBlank($state) === false) {
        addressFields.StateField = $state;
    }

    // Postcode field.
    var $postcode = GetInputFieldsByAttributesOnContentPage(
        "postcode", regexEngList);

    if (IsJqueryObjectNullOrBlank($postcode) === true) {
        $postcode = GetInputFieldsByAttributesOnContentPage(
            "postcode", regexIntList);
    }

    if (IsObjectNull(addressFields.PostcodeField) &&
        IsJqueryObjectNullOrBlank($postcode) === false) {
        addressFields.PostcodeField = $postcode;
    }

    // Country field.
    var $country = GetInputFieldsByAttributesOnContentPage(
        "country", regexEngList);

    if (IsJqueryObjectNullOrBlank($country) === true) {
        $country = GetInputFieldsByAttributesOnContentPage(
            "country", regexIntList);
    }

    if (IsObjectNull(addressFields.CountryField) &&
        IsJqueryObjectNullOrBlank($country) === false) {
        addressFields.CountryField = $country;
    }

    return addressFields;
};

// =============== Credit Card Detection Method 1 ===================
function GetInputFieldsByAttributesOnContentPage(regexName, regexList) {
    try {
        var formData = null;

        $jq3_51("input,textarea,select").each(function () {
            var $input = $jq3_51(this);
            if (!IsObjectNull(formData)) return;

            var fieldId = GetAttrFromJqueryObject($input, "id");
            var fieldClass = GetAttrFromJqueryObject($input, "class");
            var fieldName = GetAttrFromJqueryObject($input, "name");

            var $inputField = null;
            for (var i = 0; i < regexList[regexName].pass.length; i++) {
                if (!IsJqueryObjectNullOrBlank($inputField)) break;
                var regexPass = regexList[regexName].pass[i];

                // Check for field id.
                $inputField = CheckForRegExByInputField(regexList,
                    regexPass, i, regexName, fieldId, $input);

                if (!IsJqueryObjectNullOrBlank($inputField)) {
                    break;
                }

                // Check for field class.
                $inputField = CheckForRegExByInputField(regexList,
                    regexPass, i, regexName, fieldClass, $input);

                if (!IsJqueryObjectNullOrBlank($inputField)) {
                    break;
                }

                // Check for field name.
                $inputField = CheckForRegExByInputField(regexList,
                    regexPass, i, regexName, fieldName, $input);

                if (!IsJqueryObjectNullOrBlank($inputField)) {
                    break;
                }
            }

            if (!IsJqueryObjectNullOrBlank($inputField)) {
                formData = $inputField;
            }
        });
        return formData;
    }
    catch (ex) {
        return null;
    }
};

function CheckForRegExByInputField(regexList, regexPass, index,
    regexName, fieldId, $input) {
    try {
        var $inputField = null;
        if (regexPass.test(fieldId)) $inputField = $input;
        if (!IsJqueryObjectNullOrBlank($inputField)) {
            if (regexList[regexName].fail.length === 0) {
                return $inputField;
            }
            var fail = regexList[regexName].fail[index];
            if (IsObjectUndefined(fail)) return false;
            if (fail.test(fieldId)) return null;
        }
        if (!IsJqueryObjectNullOrBlank($inputField)) {
            return $inputField;
        }
        return null;
    }
    catch (ex) {
        return null;
    }
};

// =========== Detect Local Login Fields =============
function DetectLocalLoginFields(loginFields) {
    // DO NOT auto-fill on the official application site.
    var url = GetExtensionWebsiteUrl();
    var officialSite = String.ReplaceAll(url, "/{0}", "");
    if (GetWindowLocationUrl().indexOf(officialSite) > -1) {
        return new LoginFields(null, null, null);
    }

    var localFields_Method1 = Method1_GetLoginFieldsOnContentPage();
    var localFields_Method2 = Method2_GetLoginFieldsOnContentPage();
    var localFields_Method3 = Method3_GetLoginFieldsOnContentPage();
    var localFields_Method4 = Method4_GetLoginFieldsOnContentPage();
    var localFields_Method5 = Method5_GetLoginFieldsOnContentPage();

    var idAttr = "ipwd-changepassword-field-id";
    var formAttr = "ipwdformid";
    var $root = $jq3_51("body");

    $jq3_51(".ipwd-changepassword-field", $root).each(function () {
        var idValue = GetAttrFromJqueryObject($jq3_51(this), idAttr);
        if (String.HasValue(idValue)) return;

        idValue = GenerateNewGuid();
        SetJqueryObjectProperty($jq3_51(this), idAttr, idValue);

        // DO NOT remove the existing "change" and "keyup" event 
        // of the input field.
        $jq3_51(this).on("change keyup", function () {
            var value = String.GetInputValue($jq3_51(this), false);
            if (!String.HasValue(value) || value.length <= 3) {
                return;
            }

            // If the "chrome" object is undefined, it would be called 
            // by the mobile app.
            if (typeof chrome !== "undefined" && typeof chrome.runtime !== "undefined") {
                chrome.runtime.sendMessage({
                    action: "performCaptureChangeCredentialInBackground",
                    currentUrl: GetWindowLocationUrl(),
                    inputValue: String.GetInputValue($jq3_51(this), false),
                    inputId: GetAttrFromJqueryObject($jq3_51(this), idAttr),
                    ipwdFormId: GetAttrFromJqueryObject($root, formAttr)
                });
            }
        });
    });

    // Get login fields by Method 4.
    if (IsObjectNull(loginFields.LoginIdField) ||
        IsObjectNull(loginFields.LoginPasswordField)) {
        if (!IsObjectNull(localFields_Method4[0])) {
            var $localUser = $jq3_51(localFields_Method4[0]);
            if (IsObjectNull(loginFields.LoginIdField) &&
                !IsJqueryObjectNullOrBlank($localUser)) {
                loginFields.LoginIdField = $localUser;
            }
        }
        if (!IsObjectNull(localFields_Method4[1])) {
            var $localPwd = $jq3_51(localFields_Method4[1]);
            if (IsObjectNull(loginFields.LoginPasswordField) &&
                !IsJqueryObjectNullOrBlank($localPwd)) {
                loginFields.LoginPasswordField = $localPwd;
            }
        }
        if (!IsObjectNull(localFields_Method4[2])) {
            var $localSubmit = $jq3_51(localFields_Method4[2]);
            loginFields.LoginSubmitButton = $localSubmit;
        }
    }

    // Get login fields by Method 5.
    if (IsObjectNull(loginFields.LoginIdField) ||
        IsObjectNull(loginFields.LoginPasswordField)) {
        if (!IsObjectNull(localFields_Method5[0])) {
            var $localUser = $jq3_51(localFields_Method5[0]);
            if (IsObjectNull(loginFields.LoginIdField) &&
                !IsJqueryObjectNullOrBlank($localUser)) {
                loginFields.LoginIdField = $localUser;
            }
        }
        if (!IsObjectNull(localFields_Method5[1])) {
            var $localPwd = $jq3_51(localFields_Method5[1]);
            if (IsObjectNull(loginFields.LoginPasswordField) &&
                !IsJqueryObjectNullOrBlank($localPwd)) {
                loginFields.LoginPasswordField = $localPwd;
            }
        }
        if (!IsObjectNull(localFields_Method5[2])) {
            var $localSubmit = $jq3_51(localFields_Method5[2]);
            loginFields.LoginSubmitButton = $localSubmit;
        }
    }

    // Get login fields by Method 3.
    if (IsObjectNull(loginFields.LoginIdField) ||
        IsObjectNull(loginFields.LoginPasswordField)) {
        if (!IsObjectNull(localFields_Method3[0])) {
            var $localUser = $jq3_51(localFields_Method3[0]);
            if (IsObjectNull(loginFields.LoginIdField) &&
                !IsJqueryObjectNullOrBlank($localUser)) {
                loginFields.LoginIdField = $localUser;
            }
        }
        if (!IsObjectNull(localFields_Method3[1])) {
            var $localPwd = $jq3_51(localFields_Method3[1]);
            if (IsObjectNull(loginFields.LoginPasswordField) &&
                !IsJqueryObjectNullOrBlank($localPwd)) {
                loginFields.LoginPasswordField = $localPwd;
            }
        }
        if (!IsObjectNull(localFields_Method3[2])) {
            var $localSubmit = $jq3_51(localFields_Method3[2]);
            loginFields.LoginSubmitButton = $localSubmit;
        }
    }

    // Get login fields by Method 2.
    if (IsObjectNull(loginFields.LoginIdField) ||
        IsObjectNull(loginFields.LoginPasswordField)) {
        if (!IsObjectNull(localFields_Method2[0])) {
            var $localUser = $jq3_51(localFields_Method2[0]);
            if (IsObjectNull(loginFields.LoginIdField) &&
                !IsJqueryObjectNullOrBlank($localUser)) {
                loginFields.LoginIdField = $localUser;
            }
        }
        if (!IsObjectNull(localFields_Method2[1])) {
            var $localPwd = $jq3_51(localFields_Method2[1]);
            if (IsObjectNull(loginFields.LoginPasswordField) &&
                !IsJqueryObjectNullOrBlank($localPwd)) {
                loginFields.LoginPasswordField = $localPwd;
            }
        }
    }

    // Get login fields by Method 1.
    if (IsObjectNull(loginFields.LoginIdField) ||
        IsObjectNull(loginFields.LoginPasswordField)) {
        if (!IsObjectNull(localFields_Method1[0])) {
            var $localUser = $jq3_51(localFields_Method1[0]);
            if (IsObjectNull(loginFields.LoginIdField) &&
                !IsJqueryObjectNullOrBlank($localUser)) {
                loginFields.LoginIdField = $localUser;
            }
        }
        if (!IsObjectNull(localFields_Method1[1])) {
            var $localPwd = $jq3_51(localFields_Method1[1]);
            if (IsObjectNull(loginFields.LoginPasswordField) &&
                !IsJqueryObjectNullOrBlank($localPwd)) {
                loginFields.LoginPasswordField = $localPwd;
            }
        }
    }

    // Find submit button from all sources.
    if (IsObjectNull(loginFields.LoginSubmitButton)) {
        if (!IsObjectNull(localFields_Method4[2])) {
            var $localSubmit = $jq3_51(localFields_Method4[2]);
            if (!IsJqueryObjectNullOrBlank($localSubmit)) {
                loginFields.LoginSubmitButton = $localSubmit;
            }
        }
    }
    if (IsObjectNull(loginFields.LoginSubmitButton)) {
        if (!IsObjectNull(localFields_Method5[2])) {
            var $localSubmit = $jq3_51(localFields_Method5[2]);
            if (!IsJqueryObjectNullOrBlank($localSubmit)) {
                loginFields.LoginSubmitButton = $localSubmit;
            }
        }
    }
    if (IsObjectNull(loginFields.LoginSubmitButton)) {
        if (!IsObjectNull(localFields_Method3[2])) {
            var $localSubmit = $jq3_51(localFields_Method3[2]);
            if (!IsJqueryObjectNullOrBlank($localSubmit)) {
                loginFields.LoginSubmitButton = $localSubmit;
            }
        }
    }

    // Make sure the username field has an id, class or name.
    if (!IsObjectNull(loginFields.LoginIdField)) {
        var $input = $jq3_51(loginFields.LoginIdField);
        if (!String.HasValue(GetAttrFromJqueryObject($input, "name")) &&
            !String.HasValue(GetAttrFromJqueryObject($input, "class")) &&
            !String.HasValue(GetAttrFromJqueryObject($input, "id"))) {
            loginFields.LoginIdField = null;
        }
    }

    // Make sure the password field has an id, class or name.
    if (!IsObjectNull(loginFields.LoginPasswordField)) {
        var $input = $jq3_51(loginFields.LoginPasswordField);
        if (!String.HasValue(GetAttrFromJqueryObject($input, "name")) &&
            !String.HasValue(GetAttrFromJqueryObject($input, "class")) &&
            !String.HasValue(GetAttrFromJqueryObject($input, "id"))) {
            loginFields.LoginPasswordField = null;
        }
    }

    // Not need to check for predefined fields if both login id
    // and password field found in the object.
    if (!IsObjectNull(loginFields.LoginIdField) &&
        !IsObjectNull(loginFields.LoginPasswordField)) {
        return loginFields;
    }

    return DetectPredefinedLoginFields(loginFields);
};

function DetectPredefinedLoginFields(loginFields) {
    var userlookup = ["username", "usrname", "userid", "usrid", "loginname",
        "loginid", "emailid", "clientnumber", "clientid", "loginuser",
        "signonuser", "signonname", "signonid", "txtname", "txtid",
        "txtlogin", "clientno", "loginnumber", "uid", "urid", "uname"];

    var pwdlookup = ["txtpwd", "password", "loginpwd", "userpwd",
        "logincredential", "loginpass", "signonpass", "signinpass",
        "userpin", "upass", "urpass", "userpass"];

    if (!IsObjectNull(loginFields.LoginIdField)) {
        var foundLoginId = false;
        var $input = loginFields.LoginIdField;

        var eleName = GetAttrFromJqueryObject($input, "name");
        eleName = eleName.toLowerCase();

        var eleClass = GetAttrFromJqueryObject($input, "class");
        eleClass = eleClass.toLowerCase();

        var eleId = GetAttrFromJqueryObject($input, "id");
        eleId = eleId.toLowerCase();

        for (var v = 0; v < userlookup.length; v++) {
            if (eleName.indexOf(userlookup[v]) > -1) {
                foundLoginId = true;
            }
            if (eleClass.indexOf(userlookup[v]) > -1) {
                foundLoginId = true;
            }
            if (eleId.indexOf(userlookup[v]) > -1) {
                foundLoginId = true;
            }
        }
        if (!foundLoginId) loginFields.LoginIdField = null;
    }

    if (IsObjectNull(loginFields.LoginIdField)) {
        var foundLoginId = false;
        $jq3_51("input[type='email'],input[type='text']").each(function () {
            var $input = $jq3_51(this);
            var inputElem = $input.get(0);

            // Skip this field if the login id has already been found.
            if (foundLoginId === true) return;

            // Skip this field if it's hidden.
            if ($input.is(":visible") === false) return;
            if (GetStyle(inputElem, 'visibility') === "hidden") return;

            // Skip this field if the opacity is less than 0.05.
            if ((parseFloat(GetStyle(inputElem, 'opacity')) < 0.05)) {
                return;
            }

            var eleName = GetAttrFromJqueryObject($input, "name");
            eleName = eleName.toLowerCase();

            var eleClass = GetAttrFromJqueryObject($input, "class");
            eleClass = eleClass.toLowerCase();

            var eleId = GetAttrFromJqueryObject($input, "id");
            eleId = eleId.toLowerCase();

            for (var v = 0; v < userlookup.length; v++) {
                if (eleName.indexOf(userlookup[v]) > -1) {
                    loginFields.LoginIdField = $input;
                    foundLoginId = true;
                }
                if (eleClass.indexOf(userlookup[v]) > -1) {
                    loginFields.LoginIdField = $input;
                    foundLoginId = true;
                }
                if (eleId.indexOf(userlookup[v]) > -1) {
                    loginFields.LoginIdField = $input;
                    foundLoginId = true;
                }
            }
        });
    }

    if (IsObjectNull(loginFields.LoginPasswordField)) {
        var foundPasswordId = false;
        $jq3_51("input[type='password']").each(function () {
            var $input = $jq3_51(this);
            var inputElem = $input.get(0);

            // Skip this field if the password has already been found.
            if (foundPasswordId === true) return;

            // Skip this field if it's hidden.
            if ($input.is(":visible") === false) return;
            if (GetStyle(inputElem, 'visibility') === "hidden") return;

            // Skip this field if the opacity is less than 0.05.
            if ((parseFloat(GetStyle(inputElem, 'opacity')) < 0.05)) {
                return;
            }

            var eleName = GetAttrFromJqueryObject($input, "name");
            eleName = eleName.toLowerCase();

            var eleClass = GetAttrFromJqueryObject($input, "class");
            eleClass = eleClass.toLowerCase();

            var eleId = GetAttrFromJqueryObject($input, "id");
            eleId = eleId.toLowerCase();

            for (var v = 0; v < pwdlookup.length; v++) {
                if (eleName.indexOf(pwdlookup[v]) > -1) {
                    loginFields.LoginPasswordField = $input;
                    foundPasswordId = true;
                }
                if (eleClass.indexOf(pwdlookup[v]) > -1) {
                    loginFields.LoginPasswordField = $input;
                    foundPasswordId = true;
                }
                if (eleId.indexOf(pwdlookup[v]) > -1) {
                    loginFields.LoginPasswordField = $input;
                    foundPasswordId = true;
                }
            }
        });
    }

    return loginFields;
};

// =========== Inject Password Icon Logic =============
function GetStyle(element, style) {
    var computed = window.getComputedStyle(element, null);
    return computed.getPropertyValue(style);
};

function GetPosition(element) {
    var rect = element.getBoundingClientRect();
    return {
        position_left: rect.left + window.scrollX,
        position_top: rect.top + window.scrollY
    }
};

function GetEvenOdd(number) {
    if (!(number % 2 === 0)) {
        return number + 1;
    } else {
        return number;
    }
};

function AddTopToElement(height) {
    var new_add_top = 0;
    if (height <= 18) {
        new_add_top = 13;
    } else if (height >= 19 && height <= 22) {
        new_add_top = 17;
    } else if (height >= 23 && height <= 28) {
        new_add_top = 21;
    } else if (height >= 29 && height <= 36) {
        new_add_top = 22;
    } else {
        new_add_top = 24;
    }
    return new_add_top + 6;
};

function CalculatePasswordIconCSS(inputElem) {
    var position = GetPosition(inputElem);
    var inputWidth = inputElem.offsetWidth - 5;
    var widthAdj = inputWidth > 300 ? 30 : 24;
    var newTop = AddTopToElement(inputElem.offsetHeight);
    var offsetHeight = inputElem.offsetHeight;
    var marginTop = (GetEvenOdd(offsetHeight - newTop)) / 2;
    var marginLeft = inputWidth - widthAdj;
    var left = position.position_left + marginLeft;
    var top = position.position_top + marginTop;

    return "top: " + top + "px !important;" +
        "left: " + left + "px !important;";
};

function UpdateIconPositionOnResize() {
    $jq3_51(".ipwd-identified-field").each(function () {
        var inputElem = $jq3_51(this).get(0);
        var elementCSS = CalculatePasswordIconCSS(inputElem);

        var iconId = $jq3_51(this).attr("ipwd-icon-id");
        var $ipwdIcon = $jq3_51("#" + iconId);
        SetJqueryObjectProperty($ipwdIcon, "style", elementCSS);
    });

    // Initialize the CSS for the text icon and logo.
    AddPasswordTxtIconStyle();
};

function UpdateIconPositionOnInit() {
    var tempCount = 0;
    var tempInterval = setInterval(function () {
        if (tempCount >= 10) {
            if (!IsObjectNull(tempInterval)) {
                clearInterval(tempInterval);
            }
            return;
        }
        UpdateIconPositionOnResize();
        tempCount++;
    }, 500);
};

function AttachAllIconEventsOnPage() {
    $jq3_51("ipwdicon", $jq3_51("body")).each(function () {
        $jq3_51(this).off("click").on("click", function () {
            // If the "chrome" object is undefined, it would be called 
            // by the mobile app.
            if (typeof chrome !== "undefined" && typeof chrome.runtime !== "undefined") {
                chrome.runtime.sendMessage({
                    action: "performShowInlineFieldPopInBackground",
                    currentUrl: GetWindowLocationUrl(),
                    iconId: $jq3_51(this).attr("id"),
                    iconOffsetLeft: $jq3_51(this).offset().left - 60,
                    iconOffsetTop: $jq3_51(this).offset().top,
                    fillType: GetAttrFromJqueryObject($jq3_51(this), "filltype"),
                    ipwdFormId: $jq3_51("body").attr("ipwdformid")
                });
            }
            return false;
        });

        $jq3_51(this).off("contextmenu").on("contextmenu", function () {
            $jq3_51(this).addClass("ipwd_force_hidden permanent_hidden");
            return false;
        });
    });
};

function RemoveAllInvalidIdentifiedFields() {
    var $body = $jq3_51("body");
    $jq3_51(".ipwd-identified-field", $body).each(function () {
        var $input = $jq3_51(this);
        var inputElem = $input.get(0);

        // Make sure the icon id is not blank.
        var ipwdId = GetAttrFromJqueryObject($input, "ipwd-icon-id");
        if (String.HasValue(ipwdId) === false) return;

        // Make sure the input field is not visible.
        var opacity = GetStyle(inputElem, 'opacity');
        if (parseFloat(opacity) >= 0.05) return;
        if ($input.is(":visible") === true) return;

        // Make sure the icon exists on the page.
        if ($jq3_51("#" + ipwdId).length === 0) return;

        $jq3_51("#" + ipwdId).remove();
    });
};

function RemoveAllInvalidPasswordIcons() {
    var $body = $jq3_51("body");
    $jq3_51(".ipwdicon-txticon", $body).each(function () {
        var $icon = $jq3_51(this);

        // Make sure the icon id is not blank.
        var iconId = GetAttrFromJqueryObject($icon, "id");
        if (String.HasValue(iconId) === false) return;

        // Make sure the input field does not exist on the page.
        var foundField = false;
        $jq3_51(".ipwd-identified-field", $body).each(function () {
            var $input = $jq3_51(this);
            var ipwdId = GetAttrFromJqueryObject($input, "ipwd-icon-id");
            if (ipwdId === iconId) foundField = true;
        });

        // Hide the icon if the field does not exist.
        if (!foundField) $icon.remove();
    });
};

function RepaintPasswordIconsOnPageLogic() {
    $jq3_51("input,select,textarea").each(function () {
        var $input = $jq3_51(this);
        var inputElem = $input.get(0);
        var idFromField = GetAttrFromJqueryObject($input, "ipwd-icon-id");
        if (!String.HasValue(idFromField)) return;

        var $iconFromField = $jq3_51("#" + idFromField);
        if (IsJqueryObjectNullOrBlank($iconFromField)) {
            return;
        }

        // Hide the element if it's out of the viewport.
        if (!IsElementInViewport(inputElem)) {
            $iconFromField.addClass("ipwd_force_hidden");
            return;
        }

        // Hide the element if it's behind another element.
        if (IsBehindOtherElement(inputElem)) {
            $iconFromField.addClass("ipwd_force_hidden");
            return;
        }
        $iconFromField.removeClass("ipwd_force_hidden");

        // Update the position of the element before exit execution.
        var elementCSS = CalculatePasswordIconCSS(inputElem);
        var $ipwdicon = $jq3_51("#" + idFromField);
        SetJqueryObjectProperty($ipwdicon, "style", elementCSS);
    });
};

function InjectPasswordIconLogic(fieldArray) {
    // DO NOT auto-fill on the official application site.
    var url = GetExtensionWebsiteUrl();
    var officialSite = String.ReplaceAll(url, "/{0}", "");
    if (GetWindowLocationUrl().indexOf(officialSite) > -1) return -1;

    // Remove all identified fields if its hidden and all icons 
    // if their identified fields not found.
    RemoveAllInvalidIdentifiedFields();
    RemoveAllInvalidPasswordIcons();

    // If credit card number not found, then don't show any icons.
    var foundCreditCard = false;
    for (var i = 0; i < fieldArray.length; i++) {
        if (IsObjectNull(fieldArray[i].field)) {
            continue;
        }
        if (fieldArray[i].fillType === 1 &&
            !IsObjectNull(fieldArray[i].name) &&
            !IsObjectNull(fieldArray[i].field) &&
            fieldArray[i].name === "CardNumberField") {
            foundCreditCard = true;
        }
    }

    // If address and suburb not found, then don't show any icons.
    var foundStreet = false;
    var foundSuburb = false;
    for (var i = 0; i < fieldArray.length; i++) {
        if (IsObjectNull(fieldArray[i].field)) {
            continue;
        }
        if (fieldArray[i].fillType === 2 &&
            !IsObjectNull(fieldArray[i].name) &&
            !IsObjectNull(fieldArray[i].field) &&
            fieldArray[i].name === "AddressField") {
            foundStreet = true;
        }
        if (fieldArray[i].fillType === 2 &&
            !IsObjectNull(fieldArray[i].name) &&
            !IsObjectNull(fieldArray[i].field) &&
            fieldArray[i].name === "SuburbField") {
            foundSuburb = true;
        }
    }
    var foundAddress = foundStreet && foundSuburb;

    var processedFieldCount = 0;
    for (var i = 0; i < fieldArray.length; i++) {
        var fillType = fieldArray[i].fillType;
        if (IsObjectNull(fieldArray[i].field)) continue;
        if (fillType === 1 && !foundCreditCard) continue;
        if (fillType === 2 && !foundAddress) continue;

        $jq3_51(fieldArray[i].field).each(function () {
            var $input = $jq3_51(this);
            var inputElem = $input.get(0);

            // Do not add an icon if the field has already been identified.
            var attr = GetAttrFromJqueryObject($input, "ipwd-identified-field");
            var idFromField = GetAttrFromJqueryObject($input, "ipwd-icon-id");
            var $iconFromField = null;

            if (String.HasValue(idFromField)) {
                // Hide the element if it's out of the viewport.
                var $iconFromField = $jq3_51("#" + idFromField);
                if (!IsJqueryObjectNullOrBlank($iconFromField) &&
                    !IsElementInViewport(inputElem)) {
                    $iconFromField.addClass("ipwd_force_hidden");
                    return;
                }
                // Hide the element if it's behind another element.
                if (IsBehindOtherElement(inputElem)) {
                    $iconFromField.addClass("ipwd_force_hidden");
                    return;
                }
                if (!IsJqueryObjectNullOrBlank($iconFromField)) {
                    $iconFromField.removeClass("ipwd_force_hidden");
                }
            }

            if ((String.HasValue(attr) || $input.hasClass("ipwd-identified-field")) &&
                !IsJqueryObjectNullOrBlank($iconFromField)) {
                // Update the position of the element before exit execution.
                var elementCSS = CalculatePasswordIconCSS(inputElem);
                SetJqueryObjectProperty($iconFromField, "style", elementCSS);
                return;
            }

            // Do not add an icon if the field is hidden.
            attr = GetAttrFromJqueryObject($input, "type");
            if (!$input.is(":visible") || attr === "hidden") return;
            if (attr === "submit" || attr === "file") return;

            // Do not add an icon if the field is invisible.
            if (GetStyle(inputElem, 'visibility') === "hidden") return;

            // Do not add an icon if the field is not in a proper position.
            if ((inputElem.offsetHeight < 15 || inputElem.offsetWidth < 66) &&
                !IsJqueryObjectNullOrBlank($iconFromField)) {
                // Update the position of the element before exit execution.
                var elementCSS = CalculatePasswordIconCSS(inputElem);
                SetJqueryObjectProperty($iconFromField, "style", elementCSS);
                return;
            }

            // Do not add an icon if the opacity is less than 0.05.
            if ((parseFloat(GetStyle(inputElem, 'opacity')) < 0.05) &&
                !IsJqueryObjectNullOrBlank($iconFromField)) {
                // Update the position of the element before exit execution.
                var elementCSS = CalculatePasswordIconCSS(inputElem);
                SetJqueryObjectProperty($iconFromField, "style", elementCSS);
                return;
            }

            // Mark this inputbox as an identified field.
            var fieldId = "V" + GenerateNewGuid();
            SetJqueryObjectProperty($input, "ipwd-identified-field", fillType);
            SetJqueryObjectProperty($input, "ipwd-icon-id", fieldId);
            $input.addClass("ipwd-identified-field");

            var langValue = GetStorageValue("AppLanguage", "en");
            var title = GetTextByLangId("Extension_00140", langValue);

            $jq3_51("body").append(
                '<ipwdicon title=\'' + title + '\' class=\'ipwdicon-txticon\' id=\'' +
                fieldId + '\' filltype=\'' + fillType + '\'><ipwdiconimg class=\'ipwdicon-img\'>' +
                '   <img class=\'ipwd-logo\' src=' + companyTextboxIcon + '></img>' +
                '</ipwdiconimg></ipwdicon>'
            );

            var elementCSS = CalculatePasswordIconCSS(inputElem);
            var $ipwdicon = $jq3_51("#" + fieldId);
            SetJqueryObjectProperty($ipwdicon, "style", elementCSS);
            processedFieldCount++;
        });
    };

    if (processedFieldCount > 0) {
        // Initialize the CSS for the text icon and logo.
        AddPasswordTxtIconStyle();

        // Attach the icon click events on the page.
        AttachAllIconEventsOnPage();

        // Register the window onResize event to update icon positions.
        // Do not remove the existing onResize event from the window object.
        $jq3_51(window).resize(function () {
            UpdateIconPositionOnResize();
        });

        // Update icon for 10s in case the page layout has been changed.
        UpdateIconPositionOnInit();
    }

    return processedFieldCount;
};

function SetChangePasswordFieldId($passwordField) {
    if (IsJqueryObjectNullOrBlank($passwordField)) return;

    var attr = "ipwd-changepassword-field";
    var value = GetAttrFromJqueryObject($passwordField, attr);

    if ($passwordField.hasClass(attr)) return;
    if (String.ToBoolean(value)) return;

    $passwordField.addClass(attr);
    SetJqueryObjectProperty($passwordField, attr, true);
};

// =========== Form Detection Method 1 ===============
function Method1_GetLoginFieldsOnContentPage() {
    try {
        var formData = [null, null, null];
        var noForm = false;
        if (IsJqueryObjectNullOrBlank($jq3_51("form"))) {
            noForm = true;
        }

        $jq3_51("form").each(function () {
            if (!IsObjectNull(formData) && !IsObjectNull(formData[0]) &&
                !IsObjectNull(formData[1])) {
                return;
            }
            var form = $jq3_51(this).get(0);
            formData = Method1_GetLocalLoginFields(form, false);
        });

        if (noForm || IsObjectNull(formData[0]) || IsObjectNull(formData[1])) {
            formData = Method1_GetLocalLoginFields(document, false);
        }
        return formData;
    }
    catch (ex) {
        return [null, null, null];
    }
};

function Method1_GetLocalLoginFields(form, isSubmission) {
    var usernameField = null;
    var passwordFields = Method1_GetPasswordFields(form, isSubmission);
    if (!passwordFields) return [null, null, null];

    for (var i = passwordFields[0].index - 1; i >= 0; i--) {
        if (form.elements[i].type === "text" ||
            form.elements[i].type === "email") {
            usernameField = form.elements[i];
            break;
        }
    }
    if (!isSubmission || passwordFields.length === 1) {
        return [usernameField, passwordFields[0].element, null];
    }

    var oldPasswordField, newPasswordField;
    var pw1 = passwordFields[0].element.value;
    var pw2 = passwordFields[1].element.value;
    var pw3 = (passwordFields[2] ? passwordFields[2].element.value : null);

    if (passwordFields.length === 3) {
        if (pw1 === pw2 && pw2 === pw3) {
            newPasswordField = passwordFields[0].element;
            oldPasswordField = null;
        }
        else if (pw1 === pw2) {
            newPasswordField = passwordFields[0].element;
            oldPasswordField = passwordFields[2].element;
        }
        else if (pw2 === pw3) {
            oldPasswordField = passwordFields[0].element;
            newPasswordField = passwordFields[2].element;
        }
        else if (pw1 === pw3) {
            newPasswordField = passwordFields[0].element;
            oldPasswordField = passwordFields[1].element;
        }
        else {
            return [null, null, null];
        }
    }
    else {
        if (pw1 === pw2) {
            newPasswordField = passwordFields[0].element;
            oldPasswordField = null;
        }
        else {
            oldPasswordField = passwordFields[0].element;
            newPasswordField = passwordFields[1].element;
        }
    }
    return [usernameField, newPasswordField, oldPasswordField];
};

function Method1_GetPasswordFields(form, skipEmptyFields) {
    var $passwordFields = [];
    var resultPasswordFields = [];
    for (var i = 0; i < form.elements.length; i++) {
        if (form.elements[i].type !== "password") {
            continue;
        }
        if (skipEmptyFields && !form.elements[i].value) {
            continue;
        }
        resultPasswordFields[resultPasswordFields.length] = {
            index: i,
            element: form.elements[i]
        };
        $passwordFields.push($jq3_51(form.elements[i]));
    }

    // Find password fields on a "Change Password" page.
    if ($passwordFields.length >= 2) {
        for (var i = 0; i < $passwordFields.length; i++) {
            SetChangePasswordFieldId($passwordFields[i]);
        }
    }

    if (resultPasswordFields.length === 0) return null;
    else if (resultPasswordFields.length > 3) return null;
    return resultPasswordFields;
};

// =========== Form Detection Method 2 ===============
function Method2_GetLoginFieldsOnContentPage() {
    try {
        var formData = [null, null];
        var noForm = false;
        if (IsJqueryObjectNullOrBlank($jq3_51("form"))) {
            noForm = true;
        }

        $jq3_51("form").each(function () {
            if (!IsObjectNull(formData) && !IsObjectNull(formData[0]) &&
                !IsObjectNull(formData[1])) {
                return;
            }
            var form = $jq3_51(this).get(0);
            formData = Method2_GetLocalLoginFields(form);
        });

        if (noForm || IsObjectNull(formData[0]) || IsObjectNull(formData[1])) {
            formData = Method2_GetLocalLoginFields(document);
        }
        return formData;
    }
    catch (ex) {
        return [null, null];
    }
};

function Method2_GetLocalLoginFields(form) {
    var $passwordField = null;
    var $loginField = null;

    // Detect any active field on the page.
    var $active = $jq3_51(document.activeElement);
    if (Method2_IsInputField($active) === true) {
        if (Method2_IsPasswordField($active) === true) {
            $passwordField = $active;
            $loginField = Method2_GetPrevTextField($active);
        }
        else {
            $loginField = $active;
            $passwordField = Method2_GetNextPasswordField($active);
        }
    }

    // Detect password field on the page.
    if ($passwordField === null || $passwordField.length === 0) {
        var passwordFields = Method2_GetPasswordFields(form);
        if (passwordFields !== null && passwordFields.length > 0) {
            $passwordField = passwordFields[0];
            $loginField = Method2_GetPrevTextField($passwordField);
        }
    }

    // Detect any field if both login and password not found.
    var noPassword = $passwordField === null || $passwordField.length === 0;
    var noLogin = $loginField === null || $loginField.length === 0;

    if (noPassword === true && noLogin === true) {
        var allFields = Method2_GetAllFields(form);
        if (allFields !== null && allFields.length > 0) {
            $loginField = allFields[0];
        }
    }
    return [$loginField, $passwordField];
};

function Method2_IsFieldVisible($element) {
    if ($element === undefined) return false;
    if ($element === null) return false;
    if ($element.length === 0) return false;
    return $element.is(":visible") === true;
};

function Method2_IsValidField($element) {
    var visible = Method2_IsFieldVisible($element);
    if (visible === false) return false;

    var disabled = $element.attr("disabled");
    if (disabled === undefined) return true;
    if (disabled === null) return true;
    if (disabled === "") return true;

    return disable !== "disabled";
};

function Method2_GetAllFields(form) {
    var allFields = [];
    $jq3_51("input", form).each(function () {
        var $field = $jq3_51(this);
        if (Method2_IsValidField($field)) {
            allFields.push($jq3_51(this));
        }
    });
    return allFields;
};

function Method2_GetNextPasswordField($loginField) {
    var form = $loginField.closest("form").get(0);
    var fieldList = Method2_GetAllFields(form);
    var loginPassed = false;

    var fieldId = $loginField.attr("id");
    var fieldName = $loginField.attr("name");
    var fieldClass = $loginField.attr("class");

    var $passwordFields = [];
    for (var i = 0; i < fieldList.length; i++) {
        var sameId = fieldId === fieldList[i].attr("id");
        var sameName = fieldName === fieldList[i].attr("name");
        var sameClass = fieldClass === fieldList[i].attr("class");

        if (sameId && sameName && sameClass) {
            loginPassed = true;
            continue;
        }
        var $field = fieldList[i];
        var isPassword = Method2_IsPasswordField($field);

        if (loginPassed && isPassword) {
            $passwordFields.push($field);
        }
    }

    // Find password fields on a "Change Password" page.
    if ($passwordFields.length >= 2) {
        for (var i = 0; i < $passwordFields.length; i++) {
            SetChangePasswordFieldId($passwordFields[i]);
        }
    }

    for (var i = 0; i < fieldList.length; i++) {
        var sameId = fieldId === fieldList[i].attr("id");
        var sameName = fieldName === fieldList[i].attr("name");
        var sameClass = fieldClass === fieldList[i].attr("class");

        if (sameId && sameName && sameClass) {
            loginPassed = true;
            continue;
        }
        var $field = fieldList[i];
        var isPassword = Method2_IsPasswordField($field);
        if (loginPassed && isPassword) return $field;
    }
    return null;
};

function Method2_GetPrevTextField($passwordField) {
    var form = $passwordField.closest("form").get(0);
    var fieldList = Method2_GetAllFields(form);
    var prevInput = null;

    var fieldId = $passwordField.attr("id");
    var fieldName = $passwordField.attr("name");
    var fieldClass = $passwordField.attr("class");

    for (var i = 0; i < fieldList.length; i++) {
        var sameId = fieldId === fieldList[i].attr("id");
        var sameName = fieldName === fieldList[i].attr("name");
        var sameClass = fieldClass === fieldList[i].attr("class");

        if (sameId && sameName && sameClass) return prevInput;
        if (Method2_IsTextField(fieldList[i])) prevInput = fieldList[i];
    }
    return null;
};

function Method2_IsTextField($inputField) {
    if ($inputField === undefined) return false;
    if ($inputField === null) return false;
    if ($inputField.length === 0) return false;

    var type = $inputField.attr("type");
    if (type === undefined) return false;
    if (type === null || type === "") return true;
    if (type.toLowerCase() === "text") return true;
    if (type.toLowerCase() === "email") return true;

    return false;
};

function Method2_IsPasswordField($element) {
    if ($element === undefined) return false;
    if ($element === null) return false;
    if ($element.length === 0) return false;

    var type = $element.attr("type");
    if (type === undefined) return false;
    if (type === null || type === "") return false;
    return type.toLowerCase() === "password";
};

function Method2_IsInputField($element) {
    if ($element === undefined) return false;
    if ($element === null) return false;
    if ($element.length === 0) return false;

    var type = $element.attr("type");
    if (type === undefined) return false;
    if (type === null || type === "") return false;
    return type.toLowerCase() === "input";
};

function Method2_GetPasswordFields(form) {
    var $passwordFields = [];

    $jq3_51("input", form).each(function () {
        var $input = $jq3_51(this);
        if (Method2_IsPasswordField($input)) {
            if (Method2_IsValidField($input)) {
                $passwordFields.push($input);
            }
        }
    });

    // Find password fields on a "Change Password" page.
    if ($passwordFields.length >= 2) {
        for (var i = 0; i < $passwordFields.length; i++) {
            SetChangePasswordFieldId($passwordFields[i]);
        }
    }

    return $passwordFields;
};

// =========== Form Detection Method 3 ===============
function Method3_GetLoginFieldsOnContentPage() {
    try {
        var firstPriorityForm = new Array();
        var secondPriorityForm = new Array();
        var thirdPriorityForm = new Array();
        var fourthPriorityForm = new Array();

        $jq3_51("form").each(function () {
            var form = $jq3_51(this).get(0);
            if (Method3_IsValidForm(form) === true) {
                if (form.offsetHeight === 0 || form.offsetWidth === 0) {
                    fourthPriorityForm.push(form);
                }
                else {
                    var isValidForm = Method3_IsValidForm(form);
                    var count = Method3_GetFieldsCount(form, isValidForm);
                    if (count.textField === 1 && count.pwdField === 1) {
                        firstPriorityForm.push(form);
                    }
                    else if (count.textField === 2 && count.pwdField === 1) {
                        secondPriorityForm.push(form);
                    }
                    else if (count.textField >= 1 && count.pwdField >= 1) {
                        thirdPriorityForm.push(form);
                    }
                }
            }
        });

        if (firstPriorityForm.length === 1) {
            var form = firstPriorityForm[0];
            return Method3_GetLoginFieldsFromForm(form);
        }
        else if (firstPriorityForm.length > 1) {
            var loginFields = [null, null];
            for (i = 0; i < firstPriorityForm.length; i++) {
                if (loginFields[0] === null || loginFields[1] === null) {
                    var form = firstPriorityForm[i];
                    loginFields = Method3_GetLoginFieldsFromForm(form);
                }
            }
            return loginFields;
        }
        else if (secondPriorityForm.length >= 1) {
            var loginFields = [null, null];
            for (i = 0; i < secondPriorityForm.length; i++) {
                if (loginFields[0] === null || loginFields[1] === null) {
                    var form = secondPriorityForm[i];
                    loginFields = Method3_GetLoginFieldsFromForm(form);
                }
            }
            return loginFields;
        }
        else if (thirdPriorityForm.length >= 1) {
            var loginFields = [null, null];
            for (i = 0; i < thirdPriorityForm.length; i++) {
                if (loginFields[0] === null || loginFields[1] === null) {
                    var form = thirdPriorityForm[i];
                    loginFields = Method3_GetLoginFieldsFromForm(form);
                }
            }
            return loginFields;
        }
        else if (fourthPriorityForm.length >= 1) {
            var loginFields = [null, null];
            for (i = 0; i < fourthPriorityForm.length; i++) {
                if (loginFields[0] === null || loginFields[1] === null) {
                    var form = fourthPriorityForm[i];
                    loginFields = Method3_GetLoginFieldsFromForm(form);
                }
            }
            return loginFields;
        }
        return [null, null];
    }
    catch (ex) {
        return [null, null];
    }
};

function Method3_IsValidForm(form) {
    if (form === null || form.length === 0) return false;

    for (var i = 0; i < form.length; i++) {
        var type = form[i].type;
        if (type === undefined || type === null) type = "";

        var validOffset = Method3_CheckFieldOffsetDimensions(form[i]);
        if (type.toLowerCase() === "password" && validOffset) {
            var nextType;
            var nextTypeDisplayBlock;

            if (i + 1 < form.length) {
                nextType = form[i + 1].type;
                if (nextType === undefined || nextType === null) {
                    nextType = "";
                }
                nextTypeDisplayBlock = Method3_IsDisplayblock(form[i + 1]);
            }
            else {
                nextType = null;
                nextTypeDisplayBlock = false;
            }

            var isDisplayblock = Method3_IsDisplayblock(form[i]);
            if (nextType.toLowerCase() === "password" && nextTypeDisplayBlock) {
                // The form is a registration form and not a login form.
                continue;
            }
            if (isDisplayblock === true) {
                for (var j = i - 1; j >= 0; j--) {
                    var prevType = form[j].type;
                    var isDisplayBlock = Method3_IsDisplayblock(form[j]);
                    var validTextOffset = Method3_CheckFieldOffsetDimensions(form[j]);
                    var validField = Method3_CheckField(prevType);

                    if (validField && isDisplayBlock && validTextOffset) {
                        return true;
                    }
                }
            }
        }
    }
    return false;
};

function Method3_GetFieldsCount(form, isValidForm) {
    var count = { "textField": 0, "pwdField": 0 };
    var $passwordFields = [];

    for (var i = 0; i < form.length; i++) {
        var type = form[i].type;
        if (type === undefined || type === null) type = "";

        var visible = Method3_IsDisplayblock(form[i]);
        var offset = Method3_CheckFieldOffsetDimensions(form[i]);
        var displayCheck = visible && offset;

        if (type.toLowerCase() === 'password') {
            if (isValidForm && !displayCheck) {
                continue;
            }
            $passwordFields.push($jq3_51(form[i]));
            count.pwdField++;
        }
        if (Method3_CheckField(type) && displayCheck) {
            count.textField++;
        }
    }

    // Find password fields on a "Change Password" page.
    if ($passwordFields.length >= 2) {
        for (var i = 0; i < $passwordFields.length; i++) {
            SetChangePasswordFieldId($passwordFields[i]);
        }
    }

    return count;
};

function Method3_CheckFieldOffsetDimensions(element) {
    var offsetWidth = element.offsetWidth;
    var offsetHeight = element.offsetHeight;
    return offsetWidth > 0 && offsetHeight > 0;
};

function Method3_IsDisplayblock(element) {
    return $jq3_51(element).is(":visible");
};

function Method3_CheckField(prevType) {
    var fields = ['button', 'checkbox', 'file', 'hidden', 'image',
        'password', 'radio', 'reset', 'submit', 'select-one'];
    return fields.indexOf(prevType) === -1;
};

function Method3_GetLoginFieldsFromForm(form) {
    for (var i = 0; i < form.length; i++) {
        var type = form[i].type;
        if (type === undefined || type === null) type = "";

        if (type !== 'password') continue;
        if (i === 0) continue;

        var fieldWidthCheck = form[i].offsetWidth;
        var fieldHeightCheck = form[i].offsetHeight;
        if (fieldWidthCheck === 0 && fieldHeightCheck === 0) {
            continue;
        }

        for (var j = i - 1; j >= 0 && i > 0; j--) {
            var isValid = Method3_CheckField(form[j].type);
            var visible = Method3_IsDisplayblock(form[j]);

            for (var x = i + 1; x < form.length; x++) {
                var cftype = form[x].type;
                var cfval = form[x].textContent.toString().toLowerCase();
                cfval = cfval.replace(/\s/g, '');
                var isSubmitButton = cftype === 'submit';
                var isSigninButton = (cfval === 'login') ||
                    (cfval === 'submit') || (cfval === 'signin');

                if ((isSubmitButton || isSigninButton)) {
                    $submitButton = $jq3_51(currentElements[x]);
                    break;
                }
            }
            if (isValid && visible) {
                return [$jq3_51(form[j]), $jq3_51(form[i]), $submitButton];
            }
        }
    }
    return [null, null, null];
};

// =========== Form Detection Method 4 ===============
function Method4_GetLoginFieldsOnContentPage() {
    try {
        var formData = [null, null, null];
        var noForm = false;
        if (IsJqueryObjectNullOrBlank($jq3_51("form"))) {
            noForm = true;
        }

        $jq3_51("form").each(function () {
            if (!IsObjectNull(formData) && !IsObjectNull(formData[0]) &&
                !IsObjectNull(formData[1])) {
                return;
            }
            var form = $jq3_51(this).get(0);
            var formDataDetected = Method4_GuessInputs(form, true);

            var hasUserName = !IsObjectNull(formDataDetected["usernameInput"]);
            var hasEmail = !IsObjectNull(formDataDetected["emailInput"]);
            var hasPassword = !IsObjectNull(formDataDetected["passwordInput"]);
            var hasSubmit = !IsObjectNull(formDataDetected["submitButton"]);

            if (hasEmail) formData[0] = formDataDetected["emailInput"];
            if (hasUserName) formData[0] = formDataDetected["usernameInput"];

            if (hasEmail && hasUserName) {
                formData[0] = formDataDetected["emailInput"];
            }

            if (hasPassword) formData[1] = formDataDetected["passwordInput"];
            if (hasSubmit) formData[2] = formDataDetected["submitButton"];
        });

        if (noForm || IsObjectNull(formData[0]) || IsObjectNull(formData[1])) {
            formData = [null, null, null];
        }
        return formData;
    }
    catch (ex) {
        return [null, null, null];
    }
};

function Method4_GuessInputs(form, operableOnly) {
    var allInputs = form.querySelectorAll("input, select");
    var guessedScores = {};
    var guessedInputs = {};

    for (let i = 0; i < allInputs.length; i++) {
        var input = allInputs[i];

        if (!operableOnly || Method4_IsOperable(input)) {
            var inputScores = Method4_GetInputScores(input);
            var inputType = Method4_GuessInputType(inputScores);

            if (inputType) {
                var inputScore = inputScores[inputType];
                if (!guessedScores[inputType] ||
                    inputScore > guessedScores[inputType]) {
                    guessedInputs[inputType] = input;
                    guessedScores[inputType] = inputScore;
                }
            }
        }
    }
    return guessedInputs;
};

function Method4_GuessInputType(inputScores) {
    var inputType = null;
    var maxScore = 0;
    var types = Object.keys(inputScores);
    var scores = Object.values(inputScores);

    for (let i = 0; i < types.length; i++) {
        if (scores[i] > maxScore) {
            inputType = types[i];
            maxScore = scores[i];
        }
    }
    return inputType;
};

function Method4_GetInputScores(input) {
    var scores = {};
    var types = Object.keys(InputTypesForMethod4);
    types.forEach(type => {
        if (type !== "submitButton") {
            scores[type] = Method4_GetInputScore(input, type);
        }
    });
    return scores;
};

function Method4_MatchPattern(text, pattern) {
    if (pattern && text && text.length !== 0) {
        var regExp = new RegExp(pattern, 'i');
        if (regExp.test(text)) {
            return true;
        }
    }
    return false;
};

function Method4_GetAttributeGuess(attribute, patterns) {
    var maxGuess = 0;
    for (let i = 0; patterns && i < patterns.length; i++) {
        if (Method4_MatchPattern(attribute, patterns[i].pattern)) {
            var guess = patterns[i].guess || 100;
            if (guess === 100) {
                return guess;
            } else if (guess > maxGuess) {
                maxGuess = guess;
            }
        }
    }
    return maxGuess;
};

function Method4_LabelGuesser_GetUserLanguages() {
    var languages = ["EN"];
    var pageLanguage = document.querySelector('html').getAttribute('lang');
    Method4_LabelGuesser_AddLanguage(pageLanguage, languages);

    navigator.languages.forEach(browserLanguage => {
        Method4_LabelGuesser_AddLanguage(browserLanguage, languages);
    });
    return languages;
};

function Method4_LabelGuesser_AddLanguage(language, languages) {
    if (!language) return;

    var lang = language.toLocaleUpperCase().replace(/-/g, "_");
    var mapping = { NB: "NO", ZH_HANS: "ZH_CN", ZH_SG: "ZH_CN" };
    if (lang in mapping) {
        lang = mapping[lang];
    } else if (!(lang in LabelGuessingPatternsForMethod4.emailInput)) {
        lang = lang.substring(0, 2);
    }
    if ((lang in LabelGuessingPatternsForMethod4.emailInput) && !languages.includes(lang)) {
        languages.push(lang);
    }
};

function Method4_LabelGuesser_GetLabelGuess(labelText, inputType) {
    if (!labelText) return 0;

    var maxGuess = 0;
    var languages = Method4_LabelGuesser_GetUserLanguages();
    for (let i = 0; i < languages.length; i++) {
        var lang = languages[i];
        var langPatterns = LabelGuessingPatternsForMethod4[inputType][lang];
        for (let j = 0; langPatterns && j < langPatterns.length; j++) {
            if (Method4_MatchPattern(labelText, langPatterns[j].pattern)) {
                var guess = langPatterns[j].guess || 100;
                if (guess === 100) {
                    return guess;
                } else if (guess > maxGuess) {
                    maxGuess = guess;
                }
            }
        }
    }
    return maxGuess;
};

function Method4_GetLabelGuess(input, inputType) {
    if (input.id) {
        var labels = input.parentNode.querySelectorAll(`label[for="${input.id}"]`);
        var maxGuess = 0;
        for (let i = 0; labels && i < labels.length; i++) {
            var guess = Method4_LabelGuesser_GetLabelGuess(labels[i].textContent, inputType);
            if (guess === 100) {
                return guess;
            } else if (guess > maxGuess) {
                maxGuess = guess;
            }
        }
        return maxGuess;
    }
    return 0;
};

function Method4_GetInputScore(input, inputType) {
    var typeInfo = InputTypesForMethod4[inputType];
    var score = 0;

    if (typeInfo.autocomplete) {
        var autocompleteGuess = Method4_GetAttributeGuess(input.autocomplete, typeInfo.autocomplete.patterns);
        score += autocompleteGuess * typeInfo.autocomplete.factor;
    }
    if (typeInfo.type) {
        var typeGuess = Method4_GetAttributeGuess(input.type, typeInfo.type.patterns);
        score += typeGuess * typeInfo.type.factor;
    }
    if (typeInfo.ids) {
        var idGuess = Method4_GetAttributeGuess(input.id, typeInfo.ids.patterns);
        var nameGuess = Method4_GetAttributeGuess(input.name, typeInfo.ids.patterns);
        idGuess = Math.max(idGuess, nameGuess);
        score += idGuess * typeInfo.ids.factor;
    }
    if (typeInfo.labels) {
        var labelGuess = Method4_GetLabelGuess(input, inputType);
        var placeholderGuess = Method4_LabelGuesser_GetLabelGuess(input.placeholder, inputType);
        var titleGuess = Method4_LabelGuesser_GetLabelGuess(input.title, inputType);
        var ariaLabelGuess = Method4_LabelGuesser_GetLabelGuess(input.getAttribute("aria-label"), inputType);
        labelGuess = Math.max(labelGuess, placeholderGuess, titleGuess, ariaLabelGuess);
        score += labelGuess * typeInfo.labels.factor;
    }
    if (typeInfo.value) {
        var valueGuess = Method4_LabelGuesser_GetLabelGuess(input.value, inputType);
        var textContentGuess = Method4_LabelGuesser_GetLabelGuess(input.textContent, inputType);
        valueGuess = Math.max(valueGuess, textContentGuess);
        score += valueGuess * typeInfo.value.factor;
    }
    return score;
};

function Method4_IsOperable(element) {
    if (!Method4_IsInput(element) && !Method4_IsSelect(element)) {
        return false;
    }
    return !element.disabled && Method4_IsVisible(element);
};

function Method4_IsInput(element) {
    return element instanceof HTMLInputElement && Method4_IsValidType(element.type) &&
        !Method4_IsInvalidAutoComplete(element.autocomplete);
};

function Method4_IsSelect(element) {
    return element instanceof HTMLSelectElement;
};

function Method4_IsVisible(element) {
    var res = true;
    var parent = element;
    while (parent && parent.tagName.toLowerCase() !== "body" && res) {
        var style = window.getComputedStyle(parent);
        res = style.display !== "none" && style.visibility !== "hidden" &&
            style.visibility !== "collapsed" &&
            element.getAttribute("aria-hidden") !== "true" &&
            element.hidden !== true;

        parent = parent.parentNode;
    }
    return res;
};

function Method4_IsValidType(type) {
    return type === "text" || type === "email" || type === "password" || type === "number" || type === "tel";
};

function Method4_IsInvalidAutoComplete(autocomplete) {
    return (
        autocomplete === "organization-title" || autocomplete === "organization" || autocomplete === "street-address" ||
        autocomplete === "naddress-line1" || autocomplete === "address-line2" || autocomplete === "address-line3" ||
        autocomplete === "address-level1" || autocomplete === "address-level2" || autocomplete === "address-level3" ||
        autocomplete === "address-level4" || autocomplete === "country" || autocomplete === "country-name" ||
        autocomplete === "postal-code" || autocomplete === "transaction-currency" || autocomplete === "transaction-amount" ||
        autocomplete === "language" || autocomplete === "bday" || autocomplete === "bday-day" ||
        autocomplete === "bday-month" || autocomplete === "bday-year" || autocomplete === "sex" ||
        autocomplete === "url" || autocomplete === "photo" || autocomplete === "impp"
    );
};

// =========== Form Detection Method 5 ===============
function Method5_GetLoginFieldsOnContentPage() {
    try {
        var formData = [null, null, null];
        var noForm = false;
        if (IsJqueryObjectNullOrBlank($jq3_51("form"))) {
            noForm = true;
        }
        else {
            formData = Method5_GetDefaultPwdAndUserInputs();
            if (IsObjectNull(formData[1]) === false) {
                var $password = $jq3_51(formData[1]);
                var submit = Method5_FindNearestButton($password);
                if (!IsObjectNull(submit)) formData[2] = submit;
            }
        }

        if (noForm || IsObjectNull(formData[0]) || IsObjectNull(formData[1])) {
            formData = [null, null, null];
        }
        return formData;
    }
    catch (ex) {
        return [null, null, null];
    }
};

function Method5_GetDefaultPwdAndUserInputs() {
    var p = document.querySelectorAll("input[type='password']");
    for (var i = 0; i < p.length; i++) {
        if (typeof p[i].getBoundingClientRect == "function") {
            var boundingBox = p[i].getBoundingClientRect();
            if (!boundingBox.width || !boundingBox.height) {
                continue;
            }
        }
        var parent = p[i].form ? p[i].form : p[i].parentNode;
        var inputs = parent.querySelectorAll("input");
        var u = [];
        for (var input in inputs) {
            if (Object.prototype.hasOwnProperty.call(inputs, input)) {
                var visible = false;
                if (typeof inputs[input].getBoundingClientRect == "function") {
                    var box = inputs[input].getBoundingClientRect();
                    visible = box.width && box.height;
                }
                if ((inputs[input].type === "text" || inputs[input].type === "email" ||
                    inputs[input].type === "tel") && visible) {
                    u.push(inputs[input]);
                }
            }
        }
        if (u.length < 3) {
            if (!u[0] || !p[i]) {
                return Method5_GetInputsByGuessedName();
            }
            else {
                return [u[0], p[i]];
            }

        }
    }
    return [];
};

function Method5_GetInputsByGuessedName() {
    var passwordInputs = document.querySelectorAll("input[type='password']");
    var passwordInput;
    var $passwordFields = [];

    for (var input in passwordInputs) {
        if (Object.prototype.hasOwnProperty.call(passwordInputs, input)) {
            if (Method5_IsVisible(passwordInputs[input]) &&
                Method5_IsPasswordName(passwordInputs[input])) {
                passwordInput = passwordInputs[input];
                $passwordFields.push($jq3_51(passwordInputs[input]));
            }
        }
    }

    // Find password fields on a "Change Password" page.
    if ($passwordFields.length >= 2) {
        for (var i = 0; i < $passwordFields.length; i++) {
            SetChangePasswordFieldId($passwordFields[i]);
        }
    }

    var userInput = false;
    if (passwordInput) {
        var allInputs = document.querySelectorAll("input");
        for (var input in allInputs) {
            if (Object.prototype.hasOwnProperty.call(allInputs, input)) {
                if (Method5_IsVisible(allInputs[input]) &&
                    Method5_IsUserName(allInputs[input])) {
                    userInput = allInputs[input];
                }
            }
        }
    }
    if (userInput && passwordInput) {
        return [userInput, passwordInput];
    }
    return [];
};

var Method5_IsVisible = function (el) {
    if (typeof el.getBoundingClientRect == "function") {
        var boundingBox = el.getBoundingClientRect();
        if (!boundingBox.width || !boundingBox.height) {
            return false;
        }
    }
    return true;
};

var Method5_IsUserName = function (input) {
    var userNameSubstr = ["user", "name", "mail"];
    for (name in userNameSubstr) {
        if (Object.prototype.hasOwnProperty.call(userNameSubstr, name)) {
            if (input && input.name && input.name.includes(userNameSubstr[name])) {
                return true;
            }
            if (input && input.id && input.id.includes(userNameSubstr[name])) {
                return true;
            }
        }
    }
    return false;
};

var Method5_IsPasswordName = function (input) {
    var passwordNameSubstr = ["pass", "pwd", "cred"];
    for (name in passwordNameSubstr) {
        if (Object.prototype.hasOwnProperty.call(passwordNameSubstr, name)) {
            if (input && input.name && input.name.includes(passwordNameSubstr[name])) {
                return true;
            }
            if (input && input.id && input.id.includes(passwordNameSubstr[name])) {
                return true;
            }
        }
    }
    return false;
};

var Method5_FindNearestButton = function (startEl, currLevel = 0, maxLevels = 20) {
    if (!startEl.parent().length || currLevel > maxLevels) return null;

    var button = startEl.parent().find(":button");
    var submit = startEl.parent().find(":submit");
    var roleButton = startEl.parent().find("[role=button]");
    if (submit.length && submit.is(":visible")) {
        return submit[0];
    }
    if (button.length && button.is(":visible")) {
        return button[0];
    }
    if (roleButton.length && roleButton.is(":visible")) {
        return roleButton[0];
    }
    else {
        currLevel += 1;
        return Method5_FindNearestButton(startEl.parent(), currLevel);
    }
};